## Ridge Regression and Kernel Ridge Regression

## This code is written by Davide Albanese, <albanese@fbk.eu>.
## (C) 2011 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ["ridge_base", "Ridge", "KernelRidge"]

import numpy as np
from kernel_class import *


def ridge_base(x, y, lmb):
    """Solves the equation X beta = y by computing a vector beta that
    minimize ||y - X beta||^2 + ||lambda beta||^2 where ||.|| is the L^2
    norm (X is a NxP matrix). When if N >= P the function solves 
    the normal equation (primal solution), when N < P the function 
    solves the dual solution.

    X must be centered by columns.

    :Parameters:
       x : 2d array_like object
          training data (N x P)
       y : 1d array_like object (N)
          target values
       lmb : float (> 0.0)
          lambda, regularization parameter

    :Returns:
       beta : 1d numpy array
          beta
    """

    xarr = np.asarray(x)
    yarr = np.asarray(y)
    n, p = xarr.shape

    if n >= p:
        # primal solution
        # beta = (X'X + lambda I)^-1 X'Y
        beta = np.linalg.solve(np.dot(x.T, x) + 
            lmb * np.eye(x.shape[1]), np.dot(x.T, y))
    else:
        # dual solution 
        # solve two linear equations systems:
        # (XX' + lambda I) alpha = y
        # beta = x^T alpha
        alpha = np.linalg.solve(np.dot(x, x.T) + lmb *
            np.eye(x.shape[0]), y)
        beta = np.dot(x.T, alpha)
        
        # # SVD method
        # # beta = V'(R'R + lambda I)R'Y
        # # where X = UDV and R = UD
        # u, d, v = np.linalg.svd(xarr, full_matrices=False)
        # r = np.dot(u, np.diag(d))
        # tmp = np.linalg.inv(np.dot(r.T, r) + lmb *
        #     np.eye(r.shape[0]))
        # beta = np.dot(np.dot(np.dot(v.T, tmp), r.T), y)
        
    return beta


class Ridge:
    """Ridge Regression.
    
    Solves the equation X beta = y by computing a vector beta that
    minimize ||y - X beta||^2 + ||lambda beta||^2 where ||.|| is the L^2
    norm (X is a NxP matrix). When if N >= P the function solves 
    the normal equation (primal solution), when N < P the function 
    solves the dual solution. 
    """

    def __init__(self, lmb=1.0):
        """Initialization.

        :Parameters:
           lmb : float (>= 0.0)
              regularization parameter
        """

        self._lmb = float(lmb)
        
        if self._lmb < 0:
            raise ValueError("lmb must be >= 0")

        self._beta = None
        self._beta0 = None

    def learn(self, x, y):
        """Compute the regression coefficients.

        Parameters:
           x : 2d array_like object
              training data (N, P)
           y : 1d array_like object (N)
              target values
        """

        xarr = np.array(x, dtype=np.float, copy=True)
        yarr = np.asarray(y, dtype=np.float)

        if xarr.ndim != 2:
            raise ValueError("x must be a 2d array_like object")

        if yarr.ndim != 1:
            raise ValueError("y must be an 1d array_like object")

        if xarr.shape[0] != yarr.shape[0]:
            raise ValueError("x, y shape mismatch")

        xmean =  np.mean(xarr, axis=0)
        xarr = xarr - xmean
                
        self._beta = ridge_base(xarr, yarr, self._lmb)
        self._beta0 = np.mean(y) - np.dot(xmean, self._beta)
                
    def pred(self, t):
        """Compute the predicted response.

        :Parameters:
           t : 1d or 2d array_like object ([M,] P)
              test data

        :Returns:
           p : integer or 1d numpy darray
              predicted response
        """

        if self._beta is None:
            raise ValueError("no model computed; run learn()")

        tarr = np.asarray(t, dtype=np.float)

        if tarr.ndim > 2 or tarr.ndim < 1:
            raise ValueError("t must be an 1d or a 2d array_like object")

        try:
            p = np.dot(tarr, self._beta) + self._beta0
        except ValueError:
            raise ValueError("t, beta: shape mismatch")

        return p

    def beta(self):
        """Return b1, ..., bp.
        """

        return self._beta

    def beta0(self):
        """Return b0.
        """

        return self._beta0


class KernelRidge:
    """Kernel Ridge Regression (dual).
    """

    def __init__(self, lmb=1.0, kernel=None):
        """Initialization.

        :Parameters:
           lmb : float (>= 0.0)
              regularization parameter
           kernel : None or mlpy.Kernel object.
              if kernel is None, K and Kt in .learn()
              and in .pred() methods must be precomputed kernel 
              matricies, else K and Kt must be training (resp. 
              test) data in input space.
        """

        if lmb < 0:
            raise ValueError("lmb must be >= 0")

        if kernel is not None:
            if not isinstance(kernel, Kernel):
                raise ValueError("kernel must be None or a mlpy.Kernel object")

        self._lmb = float(lmb)
        self._kernel = kernel
        self._alpha = None
        self._b = None
        self._x = None
                                
    def learn(self, K, y):
        """Compute the regression coefficients.

        Parameters:
           K: 2d array_like object
              precomputed training kernel matrix (if kernel=None);
              training data in input space (if kernel is a Kernel object)
           y : 1d array_like object (N)
              target values
        """

        K_arr = np.asarray(K, dtype=np.float)
        y_arr = np.asarray(y, dtype=np.float)

        if K_arr.ndim != 2:
            raise ValueError("K must be a 2d array_like object")

        if y_arr.ndim != 1:
            raise ValueError("y must be an 1d array_like object")
        
        if K_arr.shape[0] != y_arr.shape[0]:
            raise ValueError("K, y shape mismatch")

        if self._kernel is None:
            if K_arr.shape[0] != K_arr.shape[1]:
                raise ValueError("K must be a square matrix")
        else:
            self._x = K_arr.copy()
            K_arr = self._kernel.kernel(K_arr, K_arr)

        n = K_arr.shape[0]

        # dual solution 
        # (K + lambda I) alpha = y
        
        # Solve |K+lambdaI 1| |alpha| = |y|
        #       |    1     0| |  b  |   |0|
        # as in G. C. Cawley, N. L. C. Nicola and O. Chapelle.
        # Estimating Predictive Variances with Kernel Ridge 
        # Regression.
        A = np.empty((n+1, n+1), dtype=np.float)
        A[:n, :n] = K_arr + self._lmb * np.eye(n)
        A[n, :n], A[:n, n], A[n, n] = 1., 1., 0.
        g = np.linalg.solve(A, np.append(y_arr, 0))
        self._alpha, self._b = g[:-1], g[-1]
                
    def pred(self, Kt):
        """Compute the predicted response.

        :Parameters:
           Kt : 1d or 2d array_like object
              precomputed test kernel matrix. (if kernel=None);
              test data in input space (if kernel is a Kernel object).

        :Returns:
           p : integer or 1d numpy darray
              predicted response
        """

        if self._alpha is None:
            raise ValueError("no model computed; run learn()")

        Kt_arr = np.asarray(Kt, dtype=np.float)
        if self._kernel is not None:
            Kt_arr = self._kernel.kernel(Kt_arr, self._x)
        
        try:
            p = np.dot(self._alpha, Kt_arr.T) + self._b
        except ValueError:
            raise ValueError("Kt, alpha: shape mismatch")
    
        return p

    def alpha(self):
        """Return alpha.
        """

        return self._alpha
    
    def b(self):
        """Return b.
        """

        return self._b



import numpy as np
cimport numpy as np
cimport cython

cdef extern from "c_canberra.h":
    double c_canberra(double *x, double *y, long n)
    double c_canberra_location(long *x, long *y, long n, long k)
    double c_canberra_stability(long *x, long n, long p, long k)
    double c_canberra_expected(long n, long k)

def canberra(x, y):
    """Returns the Canberra distance between two P-vectors x and y:
    sum_i(abs(x_i - y_i) / (abs(x_i) + abs(y_i))).
    """

    cdef np.ndarray[np.float64_t, ndim=1] x_arr
    cdef np.ndarray[np.float64_t, ndim=1] y_arr
    
    x_arr = np.ascontiguousarray(x, dtype=np.float)
    y_arr = np.ascontiguousarray(y, dtype=np.float)

    if x_arr.shape[0] != y_arr.shape[0]:
        raise ValueError("x, y: shape mismatch")
    
    return c_canberra(<double *> x_arr.data, <double *> y_arr.data,
                       <long> x_arr.shape[0])


def canberra_location(x, y, k=None):
    """Returns the Canberra distance between two position lists,
    `x` and `y`. A position list of length P contains the position 
    (from 0 to P-1) of P elements. k is the location parameter,
    if k=None will be set to P.
    """

    cdef np.ndarray[np.int64_t, ndim=1] x_arr
    cdef np.ndarray[np.int64_t, ndim=1] y_arr
    
    x_arr = np.ascontiguousarray(x, dtype=np.int)
    y_arr = np.ascontiguousarray(y, dtype=np.int)

    if x_arr.shape[0] != y_arr.shape[0]:
        raise ValueError("x, y: shape mismatch")
    
    if k == None:
        k = x_arr.shape[0]

    if k <= 0 or k > x_arr.shape[0]:
        raise ValueError("k must be in [1, %i]" % x_arr.shape[0])   

    return c_canberra_location(<long *> x_arr.data,
               <long *> y_arr.data, <long> x_arr.shape[0], <long> k)


def canberra_stability(x, k=None):
    """Returns the Canberra stability indicator between N position
    lists, where `x` is an (N, P) matrix. A position list of length 
    P contains the position (from 0 to P-1) of P elements. k is 
    the location parameter, if k=None will be set to P. The lower 
    the indicator value, the higher the stability of the lists.

    The stability is computed by the mean distance of all the 
    (N(N-1))/2 non trivial values of the distance matrix (computed
    by canberra_location()) scaled by the expected (average) 
    value of the Canberra metric.

    Example:

    >>> import numpy as np
    >>> import mlpy
    >>> x = np.array([[2,4,1,3,0], [3,4,1,2,0], [2,4,3,0,1]])  # 3 position lists
    >>> mlpy.canberra_stability(x, 3) # stability indicator
    0.74862979571499755
    """

    cdef np.ndarray[np.int64_t, ndim=2] x_arr
        
    x_arr = np.ascontiguousarray(x, dtype=np.int)
    
    if k == None:
        k = x_arr.shape[1]

    if k <= 0 or k > x_arr.shape[1]:
        raise ValueError("k must be in [1, %i]" % x_arr.shape[1])
    
    return c_canberra_stability(<long *> x_arr.data, 
               <long> x_arr.shape[0], <long> x_arr.shape[1], <long> k)


def canberra_location_expected(p, k=None):
    """Returns the expected value of the Canberra location distance,
    where `p` is the number of elements and `k` is the number of 
    positions to consider.
    """

    if k == None:
        k = p

    if k <= 0 or k > p:
        raise ValueError("k must be in [1, %i]" % p)

    return c_canberra_expected(p, k)



## This code is written by Davide Albanese, <albanese@fbk.eu>
## (C) 2011 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.


import numpy as np
cimport numpy as np
from libc.stdlib cimport *

from clcs cimport *

np.import_array()


def lcs_std(x, y):
    """Standard Longest Common Subsequence (LCS)
    algorithm as described in [Cormen01]_.
    
    The elements of sequences must be coded as integers.
    
    :Parameters:
       x : 1d integer array_like object (N)
          first sequence
       y : 1d integer array_like object (M)
          second sequence

    :Returns:
       length : integer
          length of the LCS of x and y
       path : tuple of two 1d numpy array (path_x, path_y)
          path of the LCS

    Example
    
    Reproducing the example in figure 15.6 of [Cormen01]_,
    where sequence X = (A, B, C, B, D, A, B) and Y = (B, D, C, A, B, A).

    >>> import mlpy
    >>> x = [0,1,2,1,3,0,1] # (A, B, C, B, D, A, B)
    >>> y = [1,3,2,0,1,0] # (B, D, C, A, B, A)
    >>> length, path = mlpy.lcs_std(x, y)
    >>> length
    4
    >>> path
    (array([1, 2, 3, 5]), array([0, 2, 4, 5]))   

    .. [Cormen01] H Cormen et al.. Introduction to Algorithms, Second Edition. The MIT Press, 2001.
    """

    cdef np.ndarray[np.int_t, ndim=1] x_arr
    cdef np.ndarray[np.int_t, ndim=1] y_arr
    cdef np.ndarray[np.int_t, ndim=1] px_arr
    cdef np.ndarray[np.int_t, ndim=1] py_arr
    cdef char **b
    cdef int i
    cdef Path p
    cdef int length

    x_arr = np.ascontiguousarray(x, dtype=np.int)
    y_arr = np.ascontiguousarray(y, dtype=np.int)

    b = <char **> malloc ((x_arr.shape[0]+1) * sizeof(char *))
    for i in range(x_arr.shape[0]+1):
        b[i] = <char *> malloc ((y_arr.shape[0]+1) * sizeof(char))    

    length = std(<long *> x_arr.data, <long *> y_arr.data, b,
                  <int> x_arr.shape[0], <int> y_arr.shape[0])

    trace(b, <int> x_arr.shape[0], <int> y_arr.shape[0], &p)
    
    for i in range(x_arr.shape[0]+1):
        free (b[i])
    free(b)

    px_arr = np.empty(p.k, dtype=np.int)
    py_arr = np.empty(p.k, dtype=np.int)
    
    for i in range(p.k):
         px_arr[i] = p.px[i]
         py_arr[i] = p.py[i]

    free (p.px)
    free (p.py)
    
    return length, (px_arr, py_arr)


def lcs_real(x, y, eps, delta):
    """Longest Common Subsequence (LCS) for series
    composed by real numbers as described in 
    [Vlachos02]_.
       
    :Parameters:
       x : 1d integer array_like object (N)
          first sequence
       y : 1d integer array_like object (M)
          second sequence
       eps : float (>=0)
          matching threshold
       delta : int (>=0)
          controls how far in time we can go in order to
          match a given point from one series to a 
          point in another series

    :Returns:
       length : integer
          length of the LCS of x and y
       path : tuple of two 1d numpy array (path_x, path_y)
          path of the LCS

    .. [Vlachos02] M Vlachos et al.. Discovering Similar Multidimensional Trajectories. In Proceedings of the 18th international conference on data engineering, 2002
    """

    cdef np.ndarray[np.float_t, ndim=1] x_arr
    cdef np.ndarray[np.float_t, ndim=1] y_arr
    cdef np.ndarray[np.int_t, ndim=1] px_arr
    cdef np.ndarray[np.int_t, ndim=1] py_arr
    cdef char **b
    cdef int i
    cdef Path p
    cdef int length

    x_arr = np.ascontiguousarray(x, dtype=np.float)
    y_arr = np.ascontiguousarray(y, dtype=np.float)

    b = <char **> malloc ((x_arr.shape[0]+1) * sizeof(char *))
    for i in range(x_arr.shape[0]+1):
        b[i] = <char *> malloc ((y_arr.shape[0]+1) * sizeof(char))    

    length = real(<double *> x_arr.data, <double *> y_arr.data, b,
                  <int> x_arr.shape[0], <int> y_arr.shape[0],
                   <double> eps, <int> delta)

    trace(b, <int> x_arr.shape[0], <int> y_arr.shape[0], &p)
    
    for i in range(x_arr.shape[0]+1):
        free (b[i])
    free(b)

    px_arr = np.empty(p.k, dtype=np.int)
    py_arr = np.empty(p.k, dtype=np.int)
    
    for i in range(p.k):
         px_arr[i] = p.px[i]
         py_arr[i] = p.py[i]

    free (p.px)
    free (p.py)
    
    return length, (px_arr, py_arr)



## This code is written by Davide Albanese, <albanese@fbk.eu>.
##(C) 2011 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ['bootstrap_ci', 'quantile']

import numpy as np

import sys
if sys.version >= '3':
    from . import gsl
else:
    import gsl


def bootstrap_ci(x, B=1000, alpha=0.05, seed=0):
    """Computes the (1-alpha) Bootstrap confidence interval
    from empirical bootstrap distribution of sample mean. 
        
    The lower and upper confidence bounds are the (B*alpha/2)-th 
    and B * (1-alpha/2)-th ordered means, respectively.
    For B = 1000 and alpha = 0.05 these are the 25th and 975th
    ordered means.
    """
    
    x_arr = np.ravel(x)

    if B < 2:
        raise ValueError("B must be >= 2")
    
    if alpha < 0 or alpha > 1:
        raise ValueError("alpha must be in [0, 1]")

    np.random.seed(seed)
    
    bmean = np.empty(B, dtype=np.float)
    for b in range(B):
        idx = np.random.random_integers(0, x_arr.shape[0]-1, x_arr.shape[0])
        bmean[b] = np.mean(x_arr[idx])
        
    bmean.sort()
    lower = int(B * (alpha * 0.5))
    upper = int(B * (1 - (alpha * 0.5)))
    
    return (bmean[lower], bmean[upper])


def quantile(x, f):
    """Returns a quantile value of `x`.

    The quantile is determined by the `f`, a fraction between 
    0 and 1. For example, to compute the value of the 75th
    percentile `f` should have the value 0.75.
    """

    xarr = np.array(x, dtype=np.float, copy=True)
    xarr = np.ravel(x)
    xarr.sort()
    
    return gsl.stats_quantile_from_sorted_data(xarr, f)



## This code is written by Davide Albanese, <albanese@fbk.eu>.
## (C) 2010 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ['elasticnet_base', 'ElasticNet', 'ElasticNetC']

import numpy as np

def softt(beta0, lmb):
    """Soft thresholding.
    """

    lmb_half = lmb / 2.0
    idx = np.abs(beta0) < lmb_half
    beta = beta0 - np.sign(beta0) * lmb_half
    beta[idx] = 0.0

    return beta


def elasticnet_base(x, y, lmb, eps, supp=True, tol=0.01):
    """Elastic Net Regularization via Iterative Soft Thresholding.

    `x` should be centered and normalized by columns, and `y`
    should be centered.

    Computes the coefficient vector which solves the elastic-net
    regularization problem min {\|\| X beta - Y \|\|^2 + lambda(\|beta\|^2_2
    + eps \|beta\|_1}. The solution beta is computed via iterative
    soft-thresholding, with damping factor 1/(1+eps*lambda), thresholding
    factor eps*lambda, null initialization vector and step
    1 / (eig_max(XX^T)*1.1).

    :Parameters:
       x : 2d array_like object (N x P)
          matrix of regressors
       y : 1d array_like object (N)
          response
       lmb : float
          regularization parameter controlling overfitting.
          `lmb` can be tuned via cross validation.
       eps : float
          correlation parameter preserving correlation
          among variables against sparsity. The solutions
          obtained for different values of the correlation
          parameter have the same prediction properties but
          different feature representation.
       supp : bool
          if True, the algorithm stops when the support of beta
          reached convergence. If False, the algorithm stops when
          the coefficients reached convergence, that is when
          the beta_{l}(i) - beta_{l+1}(i) > tol * beta_{l}(i)
          for all i.
       tol : double
          tolerance for convergence

    :Returns:
       beta, iters : 1d numpy array, int
          beta, number of iterations performed
    """

    xarr = np.asarray(x)
    yarr = np.asarray(y)

    xt = xarr.T
    xy = np.dot(xt, yarr)

    n, p = xarr.shape
    if p >= n:
        xx = np.dot(xarr, xt)
    else:
        xx = np.dot(xt, xarr)

    step = 1. / (np.linalg.eigvalsh(xx).max() * 1.1)
    lmb = lmb * n * step
    damp = 1. / (1 + lmb * eps)
    beta0 = np.zeros(p, dtype=np.float)

    k, i = 0, 0
    kmax = 100000
    imax = 10

    tmp = beta0 + (step * (np.dot(-xt, np.dot(xarr, beta0)) + xy))
    beta = softt(tmp, lmb) * damp

    while ((k < kmax) and (i < imax)):
        if supp:
            if np.any(((beta0 != 0) - (beta != 0)) != 0):
                i = 0
        else:
            if np.any(np.abs(beta0 - beta) > tol * np.abs(beta0)):
                i = 0

        beta0 = beta
        tmp = beta0 + (step * (np.dot(-xt, np.dot(xarr, beta0)) + xy))
        beta = softt(tmp, lmb) * damp

        i, k = i+1, k+1
        imax = np.max((1000, 10**np.floor(np.log10(k))))

    return beta, k


class ElasticNet(object):
    """Elastic Net Regularization via Iterative Soft Thresholding.

    Computes the coefficient vector which solves the elastic-net
    regularization problem min {\|\| X beta - Y \|\|^2 + lambda(\|beta\|^2_2
    + eps \|beta\|_1}. The solution beta is computed via iterative
    soft-thresholding, with damping factor 1/(1+eps*lambda), thresholding
    factor eps*lambda, null initialization vector and step
    1 / (eig_max(XX^T)*1.1).
    """

    def __init__(self, lmb, eps, supp=True, tol=0.01):
        """Initialization.

        :Parameters:
            lmb : float
               regularization parameter controlling overfitting.
               `lmb` can be tuned via cross validation.
            eps : float
               correlation parameter preserving correlation
               among variables against sparsity. The solutions
               obtained for different values of the correlation
               parameter have the same prediction properties but
               different feature representation.
            supp : bool
               if True, the algorithm stops when the support of beta
               reached convergence. If False, the algorithm stops when
               the coefficients reached convergence, that is when
               the beta_{l}(i) - beta_{l+1}(i) > tol * beta_{l}(i)
               for all i.
            tol : double
               tolerance for convergence
        """

        self._lmb = float(lmb)
        self._eps = float(eps)
        self._supp = supp
        self._tol = float(tol)

        self._beta = None
        self._beta0 = None
        self._iters = None

    def learn(self, x, y):
        """Compute the regression coefficients.

        :Parameters:
          x : 2d array_like object (N x P)
            matrix of regressors
          y : 1d array_like object (N)
            response
        """

        xarr = np.array(x, dtype=np.float, copy=True)
        yarr = np.array(y, dtype=np.float, copy=True)

        if xarr.ndim != 2:
            raise ValueError("x must be a 2d array_like object")

        if yarr.ndim != 1:
            raise ValueError("y must be an 1d array_like object")

        if xarr.shape[0] != yarr.shape[0]:
            raise ValueError("x, y shape mismatch")

        # center x
        xmean = np.mean(xarr, axis=0)
        xarr -= xmean

        # normalize x
        xnorm = np.sqrt(np.sum((xarr)**2, axis=0))
        xarr /= xnorm

        # center y
        ymean = np.mean(yarr)

        self._beta, self._iters = elasticnet_base(xarr, yarr,
            lmb=self._lmb, eps=self._eps, supp=self._supp, tol=self._tol)
        self._beta /= xnorm
        self._beta0 = ymean - np.dot(xmean, self._beta)

    def pred(self, t):
        """Compute the predicted response.

        :Parameters:
           t : 1d or 2d array_like object ([M,] P)
              test data

        :Returns:
           p : float or 1d numpy array
              predicted response
        """

        if self._beta0 is None:
            raise ValueError('no mode computed; run learn() first')

        tarr = np.asarray(t, dtype=np.float)

        if tarr.ndim > 2 or tarr.ndim < 1:
            raise ValueError("t must be an 1d or a 2d array_like object")

        try:
            p = np.dot(tarr, self._beta) + self._beta0
        except ValueError:
            raise ValueError("t, beta: shape mismatch")

        return p

    def iters(self):
        """Return the number of iterations performed.
        """

        return self._iters

    def beta(self):
        """Return b_1, ..., b_p.
        """

        return self._beta

    def beta0(self):
        """Return b_0.
        """

        return self._beta0


class ElasticNetC(ElasticNet):
    """Elastic Net Regularization via Iterative Soft Thresholding
    for classification.

    See the ElasticNet class documentation.
    """

    def __init__(self, lmb, eps, supp=True, tol=0.01):
        """Initialization.

        :Parameters:
            lmb : float
               regularization parameter controlling overfitting.
               `lmb` can be tuned via cross validation.
            eps : float
               correlation parameter preserving correlation
               among variables against sparsity. The solutions
               obtained for different values of the correlation
               parameter have the same prediction properties but
               different feature representation.
            supp : bool
               if True, the algorithm stops when the support of beta
               reached convergence. If False, the algorithm stops when
               the coefficients reached convergence, that is when
               the beta_{l}(i) - beta_{l+1}(i) > tol * beta_{l}(i)
               for all i.
            tol : double
               tolerance for convergence
        """
        
        ElasticNet.__init__(self, lmb, eps, supp=True, tol=0.01)
        self._labels = None

    def learn(self, x, y):
        """Compute the classification coefficients.

        :Parameters:
          x : 2d array_like object (N x P)
            matrix
          y : 1d array_like object integer (N)
            class labels
        """

        yarr = np.asarray(y, dtype=np.int)
        self._labels = np.unique(yarr)
        
        k = self._labels.shape[0]
        if k != 2:
            raise ValueError("number of classes must be = 2")

        ynew = np.where(yarr == self._labels[0], -1, 1) 

        ElasticNet.learn(self, x, ynew)

    def pred(self, t):
        """Compute the predicted labels.

        :Parameters:
           t : 1d or 2d array_like object ([M,] P)
              test data

        :Returns:
           p : integer or 1d numpy array
              predicted labels
        """

        p = ElasticNet.pred(self, t)
        ret = np.where(p>0, self._labels[1], self._labels[0])

        return ret

    def w(self):
        """Returns the coefficients.
        """
        if ElasticNet.beta(self) is None:
            raise ValueError("No model computed")

        return super(ElasticNetC, self).beta()

    def bias(self):
        """Returns the bias.
        """
        if ElasticNet.beta0(self) is None:
            raise ValueError("No model computed")

        return super(ElasticNetC, self).beta0()

    def labels(self):
        """Outputs the name of labels.
        """
        
        return self._labels



## This code is written by Davide Albanese, <albanese@fbk.eu>.
## (C) 2011 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np

__all__ = ["Perceptron"]

class Perceptron:
    """Perceptron binary classifier.
    """

    def __init__(self, alpha=0.1, thr=0.0, maxiters=1000):
        """The algorithm stops when the iteration error is less
        or equal than `thr`, or a predetermined number of 
        iterations (`maxiters`) have been completed.

        :Parameters:
           alpha : float, in range (0.0, 1]
              learning rate
           thr : float, in range [0.0, 1.0]
              iteration error (e.g. thr=0.13 for error=13%) 
           maxiters : integer (>0)
              maximum number of iterations
        """


        self._alpha = alpha # learning rate, where 0.0 < alpha <= 1
        self._maxiters = maxiters
        self._thr = float(thr) # error threshold

        self._labels = None
        self._w = None
        self._bias = None # bias term
        self._err = None
        self._iters = None

    def learn(self, x, y):
        """Learning method.

        :Parameters:
           x : 2d array_like object
              training data (N, P)
           y : 1d array_like object integer (only two classes)
              target values (N)
        """

        xarr = np.asarray(x, dtype=np.float)
        yarr = np.asarray(y, dtype=np.int)
        
        if xarr.ndim != 2:
            raise ValueError("x must be a 2d array_like object")
        
        if yarr.ndim != 1:
            raise ValueError("y must be an 1d array_like object")
        
        if xarr.shape[0] != yarr.shape[0]:
            raise ValueError("x, y: shape mismatch")

        self._labels = np.unique(yarr)
        k = self._labels.shape[0]
        
        if k != 2:
            raise ValueError("number of classes must be = 2")
        
        ynew = np.where(yarr == self._labels[0], 0, 1)
        
        self._w = np.zeros(xarr.shape[1], dtype=np.float)
        self._bias = 0.0
        n = ynew.shape[0]
        
        for i in range(self._maxiters):
            tmp = np.where((np.dot(xarr, self._w)+self._bias)>0, 1, 0)
            err = np.sum(ynew != tmp) / float(n)
            
            if err <= self._thr:
                i = i - 1
                break
     
            diff = ynew - tmp
            self._w += self._alpha * np.dot(xarr.T, diff)
            self._bias += self._alpha * np.sum(diff)
                 
        tmp = np.where((np.dot(xarr, self._w)+self._bias)>0, 1, 0)
        err = np.sum(ynew != tmp) / float(n)

        self._err = err
        self._iters = i + 1

    def pred(self, t):
        """Prediction method.

        :Parameters:
           t : 1d or 2d array_like object
              testing data ([M,], P)
        """

        if self._w is None:
            raise ValueError("no model computed")

        tarr = np.asarray(t, dtype=np.float)
        if tarr.ndim > 2:
            raise ValueError("t must be an 1d or a 2d array_like object")
        
        try:
            tmp = np.dot(tarr, self._w) + self._bias
        except ValueError:
            raise ValueError("t, model: shape mismatch")

        return np.where(tmp>0, self._labels[1], self._labels[0]) \
            .astype(np.int)
        
    def w(self):
        """Returns the coefficients.
        """

        if self._w is None:
            raise ValueError("no model computed")

        return self._w

    def labels(self):
        """Outputs the name of labels.
        """
        
        return self._labels

    def bias(self):
        """Returns the bias."""
        
        if self._w is None:
            raise ValueError("no model computed.")

        return self._bias

    def err(self):
        """Returns the iteration error"""
        
        return self._err
    
    def iters(self):
        """Returns the number of iterations"""

        return self._iters

    



## This code is written by Davide Albanese, <albanese@fbk.eu>.
## (C) 2010 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ['borda_count']

import numpy as np

import sys
if sys.version >= '3':
    from . import cborda
else:
    import cborda


def borda_count(x, k=None):
    """Given N ranked ids lists of length P compute the number of
    extractions on top-k positions and the mean position for each id.
    Sort the element ids with decreasing number of extractions, and
    element ids with equal number of extractions will be sorted with
    increasing mean positions.
    
    :Parameters:
       x : 2d array_like object integer (N, P)
          ranked ids lists. For each list ids must be unique
          in [0, P-1].
       k : None or integer
          compute borda on top-k position (None -> k = P)
          
    :Returns:
       borda : 1d numpy array objects
          sorted-ids, number of extractions, mean positions

    Example:
    
    >>> import numpy as np
    >>> import mlpy
    >>> x = [[2,4,1,3,0], # first ranked list
    ...      [3,4,1,2,0], # second ranked list
    ...      [2,4,3,0,1], # third ranked list
    ...      [0,1,4,2,3]] # fourth ranked list
    >>> mlpy.borda_count(x=x, k=3)
    (array([4, 1, 2, 3, 0]), array([4, 3, 2, 2, 1]), array([ 1.25      ,  1.66666667,  0.        ,  1.        ,  0.        ]))

      * Id 4 is in the first position with 4 extractions and mean position 1.25.
      * Id 1 is in the first position with 3 extractions and mean position 1.67.
      * ...
    """
    
    x_arr = np.asarray(x, dtype=np.int)
    n, p = x_arr.shape

    if k == None:
        k = p
        
    if k < 1 or k > p:
        raise ValueError('k must be in [1, %d]' % p)

    ext, pos = cborda.core(x_arr, k)
        
    invpos = (pos + 1)**(-1) # avoid zero division
    idx = np.lexsort(keys=(invpos, ext))[::-1] 
    
    return idx, ext[idx], pos[idx]




## This code is written by Davide Albanese, <albanese@fbk.eu>
## (C) 2011 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.


import numpy as np
cimport numpy as np
from libc.stdlib cimport *
from cdtw cimport *

np.import_array()


def dtw_std(x, y, dist_only=True, squared=False):
    """Standard DTW as described in [Muller07]_,
    using the Euclidean distance (absolute value 
    of the difference) or squared Euclidean distance
    (as in [Keogh01]_) as local cost measure.

    :Parameters:
       x : 1d array_like object (N)
          first sequence
       y : 1d array_like object (M)
          second sequence
       dist_only : bool
          compute only the distance
       squared : bool
          squared Euclidean distance

    :Returns:
       dist : float
          unnormalized minimum-distance warp path 
          between sequences
       cost : 2d numpy array (N,M) [if dist_only=False]
          accumulated cost matrix
       path : tuple of two 1d numpy array (path_x, path_y) [if dist_only=False]
          warp path
    
    .. [Muller07] M Muller. Information Retrieval for Music and Motion. Springer, 2007.
    .. [Keogh01] E J Keogh, M J Pazzani. Derivative Dynamic Time Warping. In First SIAM International Conference on Data Mining, 2001.
    """

    cdef np.ndarray[np.float_t, ndim=1] x_arr
    cdef np.ndarray[np.float_t, ndim=1] y_arr
    cdef np.ndarray[np.float_t, ndim=2] cost_arr
    cdef np.ndarray[np.int_t, ndim=1] px_arr
    cdef np.ndarray[np.int_t, ndim=1] py_arr
    cdef Path p
    cdef double dist
    cdef int i
    cdef int sq

    x_arr = np.ascontiguousarray(x, dtype=np.float)
    y_arr = np.ascontiguousarray(y, dtype=np.float)
    cost_arr = np.empty((x_arr.shape[0], y_arr.shape[0]), dtype=np.float)

    if squared: sq = 1
    else: sq = 0

    dist = std(<double *> x_arr.data, <double *> y_arr.data, 
                <int> x_arr.shape[0], <int> y_arr.shape[0],
                <double *> cost_arr.data, sq)
    if dist_only:
        return dist
    else:
        path(<double *> cost_arr.data, <int> cost_arr.shape[0], 
              <int> cost_arr.shape[1], -1, -1, &p)
        px_arr = np.empty(p.k, dtype=np.int)
        py_arr = np.empty(p.k, dtype=np.int)
        for i in range(p.k):
            px_arr[i] = p.px[i]
            py_arr[i] = p.py[i] 
        free (p.px)
        free (p.py)
        return dist, cost_arr, (px_arr, py_arr)

    
def dtw_subsequence(x, y):
    """Subsequence DTW as described in [Muller07]_,
    assuming that the length of `y` is much larger 
    than the length of `x` and using the Manhattan 
    distance (absolute value of the difference) as 
    local cost measure.

    Returns the subsequence of `y` that are close to `x` 
    with respect to the minimum DTW distance.
    
    :Parameters:
       x : 1d array_like object (N)
          first sequence
       y : 1d array_like object (M)
          second sequence

    :Returns:
       dist : float
          unnormalized minimum-distance warp path
          between x and the subsequence of y
       cost : 2d numpy array (N,M) [if dist_only=False]
          complete accumulated cost matrix
       path : tuple of two 1d numpy array (path_x, path_y)
          warp path

    """

    cdef np.ndarray[np.float_t, ndim=1] x_arr
    cdef np.ndarray[np.float_t, ndim=1] y_arr
    cdef np.ndarray[np.float_t, ndim=2] cost_arr
    cdef np.ndarray[np.int_t, ndim=1] px_arr
    cdef np.ndarray[np.int_t, ndim=1] py_arr
    cdef Path p
    cdef int i
    
    x_arr = np.ascontiguousarray(x, dtype=np.float)
    y_arr = np.ascontiguousarray(y, dtype=np.float)
    cost_arr = np.empty((x_arr.shape[0], y_arr.shape[0]), dtype=np.float)

    subsequence(<double *> x_arr.data, <double *> y_arr.data, 
                 <int> x_arr.shape[0], <int> y_arr.shape[0],
                 <double *> cost_arr.data)
    
    idx = np.argmin(cost_arr[-1, :])
    dist = cost_arr[-1, idx]

    subsequence_path(<double *> cost_arr.data, <int> x_arr.shape[0],
                      <int> y_arr.shape[0], <int> idx, &p)
        
    px_arr = np.empty(p.k, dtype=np.int)
    py_arr = np.empty(p.k, dtype=np.int)
    
    for i in range(p.k):
        px_arr[i] = p.px[i]
        py_arr[i] = p.py[i]
            
    free (p.px)
    free (p.py)

    return dist, cost_arr, (px_arr, py_arr)



## This code is written by Davide Albanese, <davide.albanese@gmail.com>.
## (C) 2011 mlpy Developers

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ['MFastHCluster']


import numpy as np
import scipy.cluster.hierarchy as hierarchy
import fastcluster


class MFastHCluster:
    """Memory-saving Hierarchical Cluster (only euclidean distance).

    This method needs O(NP) memory for clustering of N point in R^P.
    """

    def __init__ (self, method='single'):
        """Initialization.

        :Parameters:
          method : string ('single', 'centroid', 'median', 'ward')
            the agglomeration method to be used
        """
        
        self._method = method
        self._Z = None

        self._linkage = False

    def linkage(self, x):
        """Performs hierarchical clustering.

        :Parameters:
          x : 2d array_like object (N, P)
             vector data, N observations in R^P
        """

        self._Z = fastcluster.linkage_vector(X=x, method=self._method, 
            metric='euclidean', extraarg=None)
        
    def Z(self):
        """Returns the hierarchical clustering encoded as a 
        linkage matrix. See `scipy.cluster.hierarchy.linkage`.
        """
        
        return self._Z

    def cut(self, t):
        """Cuts the tree into several groups by specifying the cut
        height.
        
        :Parameters:
          t : float
            the threshold to apply when forming flat clusters
         
        :Returns:
          clust : 1d numpy array
            group memberships. Groups are in 0, ..., N-1.
        """

        if self._Z is None:
            raise ValueError("No linkage computed")

        return hierarchy.fcluster(self._Z, t=t, criterion='distance') - 1



## This code is written by Davide Albanese, <albanese@fbk.eu>
## (C) 2011 mlpy Developers.

## See: Practical Guide to Wavelet Analysis - C. Torrence and G. P. Compo.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

from numpy import *
from .. import gsl

__all__ = ["cwt", "icwt", "autoscales", "fourier_from_scales",
           "scales_from_fourier"]


PI2 = 2 * pi


def normalization(s, dt):
    return sqrt((PI2 * s) / dt)


def morletft(s, w, w0, dt):
    """Fourier tranformed morlet function.
    
    Input
      * *s*    - scales
      * *w*    - angular frequencies
      * *w0*   - omega0 (frequency)
      * *dt*   - time step
    Output
      * (normalized) fourier transformed morlet function
    """
    
    p = 0.75112554446494251 # pi**(-1.0/4.0)
    wavelet = zeros((s.shape[0], w.shape[0]))
    pos = w > 0

    for i in range(s.shape[0]):
        n = normalization(s[i], dt)
        wavelet[i][pos] = n * p * exp(-(s[i] * w[pos] - w0)**2 / 2.0)
        
    return wavelet

    
def paulft(s, w, order, dt):
    """Fourier tranformed paul function.
    
    Input
      * *s*     - scales
      * *w*     - angular frequencies
      * *order* - wavelet order
      * *dt*    - time step
    Output
      * (normalized) fourier transformed paul function
    """
    
    p = 2.0**order / sqrt(order * gsl.sf_fact((2 * order) - 1))
    wavelet = zeros((s.shape[0], w.shape[0]))
    pos = w > 0

    for i in range(s.shape[0]):
        n = normalization(s[i], dt)
        tmp = s[i] * w[pos]
        wavelet[i][pos] = n * p * tmp**order * exp(-tmp)

    return wavelet


def dogft(s, w, order, dt):
    """Fourier tranformed DOG function.
    
    Input
      * *s*     - scales
      * *w*     - angular frequencies
      * *order* - wavelet order
      * *dt*    - time step
    Output
      * (normalized) fourier transformed DOG function
    """

    p =  - (0.0 + 1.0j)**order  / sqrt(gsl.sf_gamma(order + 0.5))    
    wavelet = zeros((s.shape[0], w.shape[0]), dtype = complex128)
    
    for i in range(s.shape[0]):
        n = normalization(s[i], dt)
        h = s[i] * w
        wavelet[i] = n * p * h**order * exp(-h**2 / 2.0)

    return wavelet


def angularfreq(N, dt):
    """Compute angular frequencies.

    :Parameters:   
       N : integer
          number of data samples
       dt : float
          time step
    
    :Returns:
        angular frequencies : 1d numpy array
    """

    # See (5) at page 64.
    
    N2 = N / 2.0
    w = empty(N)

    for i in range(w.shape[0]):
        if i <= N2:
            w[i] = (2 * pi * i) / (N * dt)
        else:
            w[i] = (2 * pi * (i - N)) / (N * dt)

    return w


def autoscales(N, dt, dj, wf, p):
     """Compute scales as fractional power of two.

     :Parameters:
        N : integer
           number of data samples
        dt : float
           time step
        dj : float
           scale resolution (smaller values of dj give finer resolution)
        wf : string
           wavelet function ('morlet', 'paul', 'dog')
        p : float
           omega0 ('morlet') or order ('paul', 'dog')
     
     :Returns:
        scales : 1d numpy array
           scales
     """
     
     if wf == 'dog':
         s0 = (dt * sqrt(p + 0.5)) / pi
     elif wf == 'paul':
         s0 = (dt * ((2 * p) + 1)) / (2 * pi)
     elif wf == 'morlet':
         s0 = (dt * (p + sqrt(2 + p**2))) / (2 * pi)
     else:
         raise ValueError('wavelet function not available')
     
     #  See (9) and (10) at page 67.

     J = floor(dj**-1 * log2((N * dt) / s0))
     s = empty(J + 1)
    
     for i in range(s.shape[0]):
         s[i] = s0 * 2**(i * dj)
    
     return s


def fourier_from_scales(scales, wf, p):
    """Compute the equivalent fourier period
    from scales.
    
    :Parameters:
       scales : list or 1d numpy array
          scales
       wf : string ('morlet', 'paul', 'dog')
          wavelet function
       p : float
          wavelet function parameter ('omega0' for morlet, 'm' for paul
          and dog)
    
    :Returns:
       fourier wavelengths
    """

    scales_arr = asarray(scales)

    if wf == 'dog':
        return  (2 * pi * scales_arr) / sqrt(p + 0.5)
    elif wf == 'paul':
        return  (4 * pi * scales_arr) / float((2 * p) + 1)
    elif wf == 'morlet':
        return  (4 * pi * scales_arr) / (p + sqrt(2 + p**2))
    else:
        raise ValueError('wavelet function not available')


def scales_from_fourier(f, wf, p):
    """Compute scales from fourier period.

    :Parameters:
       f : list or 1d numpy array
          fourier wavelengths
       wf : string ('morlet', 'paul', 'dog')
          wavelet function
       p : float
          wavelet function parameter ('omega0' for morlet, 'm' for paul
          and dog)
    
    :Returns:
       scales
    """

    f_arr = asarray(f)

    if wf == 'dog':
        return  (f_arr * sqrt(p + 0.5)) / (2 * pi)
    elif wf == 'paul':
        return (f_arr * ((2 * p) + 1)) / (4 * pi)
    elif wf == 'morlet':
        return (f_arr * (p + sqrt(2 + p**2))) / (4 * pi)
    else:
        raise ValueError('wavelet function not available')


def cwt(x, dt, scales, wf='dog', p=2):
    """Continuous Wavelet Tranform.

    :Parameters:
       x : 1d array_like object
          data
       dt : float
          time step
       scales : 1d array_like object
          scales
       wf : string ('morlet', 'paul', 'dog')
          wavelet function
       p : float
          wavelet function parameter ('omega0' for morlet, 'm' for paul
          and dog)
            
    :Returns:
       X : 2d numpy array
          transformed data
    """
   

    x_arr = asarray(x) - mean(x)
    scales_arr = asarray(scales)

    if x_arr.ndim is not 1:
        raise ValueError('x must be an 1d numpy array of list')

    if scales_arr.ndim is not 1:
        raise ValueError('scales must be an 1d numpy array of list')

    w = angularfreq(N=x_arr.shape[0], dt=dt)
        
    if wf == 'dog':
        wft = dogft(s=scales_arr, w=w, order=p, dt=dt)
    elif wf == 'paul':
        wft = paulft(s=scales_arr, w=w, order=p, dt=dt)
    elif wf == 'morlet':
        wft = morletft(s=scales_arr, w=w, w0=p, dt=dt)
    else:
        raise ValueError('wavelet function is not available')
    
    X_ARR = empty((wft.shape[0], wft.shape[1]), dtype=complex128)
        
    x_arr_ft = fft.fft(x_arr)

    for i in range(X_ARR.shape[0]):
        X_ARR[i] = fft.ifft(x_arr_ft * wft[i])
    
    return X_ARR


def icwt(X, dt, scales, wf='dog', p=2):
    """Inverse Continuous Wavelet Tranform.
    The reconstruction factor is not applied.

    :Parameters:
       X : 2d array_like object
          transformed data
       dt : float
          time step
       scales : 1d array_like object
          scales
       wf : string ('morlet', 'paul', 'dog')
          wavelet function
       p : float
          wavelet function parameter

    :Returns:
       x : 1d numpy array
          data
    """  
    
    X_arr = asarray(X)
    scales_arr = asarray(scales)

    if X_arr.shape[0] != scales_arr.shape[0]:
        raise ValueError('X, scales: shape mismatch')

    # See (11), (13) at page 68
    X_ARR = empty_like(X_arr)
    for i in range(scales_arr.shape[0]):
        X_ARR[i] = X_arr[i] / sqrt(scales_arr[i])
    
    x = sum(real(X_ARR), axis=0)
   
    return x



cdef extern from "src/ml.h":
    int TRUE
    int FALSE
    int SORT_ASCENDING
    int SORT_DESCENDING
    int DIST_SQUARED_EUCLIDEAN
    int DIST_EUCLIDEAN
    int SVM_KERNEL_LINEAR
    int SVM_KERNEL_GAUSSIAN
    int SVM_KERNEL_POLINOMIAL
    int BAGGING
    int AGGREGATE
    int ADABOOST

    ctypedef struct NearestNeighbor:
        int n
        int d
        double **x
        int *y
        int nclasses
        int *classes
        int k
        int dist
        
    int compute_nn(NearestNeighbor *nn, int n, int d, double *x[], int y[],
                   int k, int dist)
    int predict_nn(NearestNeighbor *nn, double x[], double **margin)
    
    ctypedef struct Node:
        double **data
        int *classes
        int npoints
        int nvar
        int nclasses
        int *npoints_for_class
        double *priors
        int node_class
        int terminal
        int left
        int right
        int var
        double value
        
    ctypedef struct Tree:
        int n
        int d
        double **x
        int *y
        int nclasses
        int *classes
        Node *node
        int nnodes
        int stumps
        int minsize

    int compute_tree(Tree *tree, int n, int d, double *x[], int y[],
                   int stumps, int minsize)
    int predict_tree(Tree *tree, double x[], double **margin)

    ctypedef struct MaximumLikelihood:
        int nclasses
        int *classes
        int *npoints_for_class
        int d
        double **mean
        double ***covar        
        double ***inv_covar
        double *priors
        double *det

    int compute_ml(MaximumLikelihood *ml, int n, int d, double *x[], int y[])
    int predict_ml(MaximumLikelihood *ml, double x[], double **margin)



## This code is written by Davide Albanese, <albanese@fbk.eu>.
## (C) 2009 Fondazione Bruno Kessler - Via Santa Croce 77, 38100 Trento, ITALY.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ['HCluster']


import numpy as np

import sys
if sys.version >= '3':
    from . import chc
else:
    import chc


class HCluster:
    """Hierarchical Cluster.
    """

    METHODS = {
        'ward': 1,
        'single': 2,
        'complete': 3,
        'average': 4,
        'mcquitty': 5,
        'median': 6,
        'centroid': 7
        }

    def __init__ (self, method='complete'):
        """Initialization.

        :Parameters:
          method : string ('ward', 'single', 'complete', 'average', 'mcquitty', 'median', 'centroid')
            the agglomeration method to be used
        """
        
        self.method = self.METHODS[method]
        
        self._ia = None
        self._ib = None
        self._order = None
        self._height = None

        self._linkage = False

    def linkage(self, y):
        """Performs hierarchical clustering on the condensed
        distance matrix y.

        :Parameters:
          y : 1d array_like object
            condensed distance matrix y. y must be a C(n, 2) sized 
            vector where n is the number of original observations 
            paired in the distance matrix. 
        """

        y_a = np.asarray(y)
        l = y.shape[0]

        if l == 0:
            raise ValueError("invalid condensed distance matrix y")

        n = int(np.ceil(np.sqrt(l * 2)))

        if (n*(n-1)/2) != l:
            raise ValueError("invalid condensed distance matrix y")

        self._ia, self._ib, self._order, self._height = \
            chc.linkage(n, y, self.method)
        
        self._linkage = True

    def cut(self, t):
        """Cuts the tree into several groups by specifying the cut
        height.
        
        :Parameters:
          t : float
            the threshold to apply when forming flat clusters
         
        :Returns:
          clust : 1d numpy array
            group memberships. Groups are in 0, ..., N-1.
        """

        if self._linkage == False:
            raise ValueError("No linkage computed")

        return chc.cut(self._ia, self._ib, self._height, t) - 1



## This code is written by Davide Albanese, <albanese@fbk.eu>.
## (C) 2010 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np

__all__ = ['uwt_align_h2', 'uwt_align_d4']


def uwt_align_h2(X, inverse=False):
    """UWT h2 coefficients aligment.

    If inverse = True performs the misalignment
    for a correct reconstruction.
    """
    
    J = X.shape[0] / 2
    shifts = np.asarray([2**j for j in range(J)])
    
    if not inverse:
        shifts *= -1

    for j in range(J):
        X[j] = np.roll(X[j], shifts[j])
        X[j+J] = np.roll(X[j+J], shifts[j])
    

def uwt_align_d4(X, inverse=False):
    """UWT d4 coefficients aligment.

    If inverse = True performs the misalignment
    for a correct reconstruction.
    """
    J = X.shape[0] / 2
    w_shifts = np.asarray([(3 * 2**j) - 1 for j in range(J)])
    v_shifts = np.asarray([1] + [(2**(j+1) - 1) for j in range(1, J)])
    
    if not inverse:
        w_shifts *= -1
        v_shifts *= -1

    for j in range(J):
        X[j] = np.roll(X[j], w_shifts[j])
        X[j+J] = np.roll(X[j+J], v_shifts[j])



## This code is written by Davide Albanese, <albanese@fbk.eu>.
## (C) 2008 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ['error', 'error_p', 'error_n', 'accuracy', 
           'sensitivity', 'specificity', 'ppv', 'npv', 
           'mcc', 'auc_wmw', 'mse', 'r2', 'r2_corr']


import numpy as np

"""
Compute metrics for assessing the performance of
classification/regression models.

The Confusion Matrix:

Total Samples       (ts) | Actual Positives (ap) | Actual Negatives (an)
------------------------------------------------------------------------
Predicted Positives (pp) | True Positives   (tp) | False Positives  (fp)
------------------------------------------------------------------------
Predicted Negatives (pn) | False Negatives  (fn) | True Negatives   (tn)
"""


def is_binary(x):
    
    ux = np.unique(x)
    for elem in ux:
        if elem not in [-1, 1]:
            return False
    return True

def true_pos(t, p):
    w = (t == 1)
    return (t[w] == p[w]).sum()

def true_neg(t, p):
    w = (t == -1)
    return (t[w] == p[w]).sum()

def false_pos(t, p):
    w = (t == -1)
    return (t[w] != p[w]).sum()

def false_neg(t, p):
    w = (t == 1)
    return (t[w] != p[w]).sum()


def error(t, p):
    """Error for binary and multiclass classification
    problems.

    :Parameters:
      t : 1d array_like object integer
        target values
      p : 1d array_like object integer
        predicted values

    :Returns:
      error : float, in range [0.0, 1.0]
    """
  
    tarr = np.asarray(t, dtype=np.int)
    parr = np.asarray(p, dtype=np.int)

    if tarr.shape[0] != parr.shape[0]:
        raise ValueError("t, p: shape mismatch")
        
    return (tarr != parr).sum() / float(tarr.shape[0])


def accuracy(t, p):
    """Accuracy for binary and multiclass classification
    problems.

    :Parameters:
      t : 1d array_like object integer
        target values
      p : 1d array_like object integer
        predicted values

    :Returns:
      accuracy : float, in range [0.0, 1.0]
    """
  
    tarr = np.asarray(t, dtype=np.int)
    parr = np.asarray(p, dtype=np.int)

    if tarr.shape[0] != parr.shape[0]:
        raise ValueError("t, p: shape mismatch")
        
    return (tarr == parr).sum() / float(tarr.shape[0])


def error_p(t, p):
    """Compute the positive error as:

    error_p = fn / ap
    
    Only binary classification problems with t[i] = -1/+1
    are allowed.

    :Parameters:
      t : 1d array_like object integer (-1/+1)
        target values
      p : 1d array_like object integer (-1/+1)
        predicted values

    :Returns:
      errorp : float, in range [0.0, 1.0]
    """

    tarr = np.asarray(t, dtype=np.int)
    parr = np.asarray(p, dtype=np.int)

    if tarr.shape[0] != parr.shape[0]:
        raise ValueError("t, p: shape mismatch")
    
    if not is_binary(tarr):
        raise ValueError("only binary classification problems"
            " with t[i] = -1/+1 are allowed.")

    fn = false_neg(tarr, parr)
    ap = float((true_pos(tarr, parr) + fn))
    
    if ap == 0:
        return 0.0

    return fn / ap


def error_n(t, p):
    """Compute the negative error as:

    error_n = fp / an
    
    Only binary classification problems with t[i] = -1/+1
    are allowed.

    :Parameters:
      t : 1d array_like object integer (-1/+1)
        target values
      p : 1d array_like object integer (-1/+1)
        predicted values

    :Returns:
      errorp : float, in range [0.0, 1.0]
    """

    tarr = np.asarray(t, dtype=np.int)
    parr = np.asarray(p, dtype=np.int)

    if tarr.shape[0] != parr.shape[0]:
        raise ValueError("t, p: shape mismatch")
    
    if not is_binary(tarr):
        raise ValueError("only binary classification problems"
            " with t[i] = -1/+1 are allowed.")

    fp = false_pos(tarr, parr)
    an = float((true_neg(tarr, parr) + fp))
    
    if an == 0:
        return 0.0

    return fp / an


def sensitivity(t, p):
    """Sensitivity, computed as:

    sensitivity = tp / ap
    
    Only binary classification problems with t[i] = -1/+1
    are allowed.

    :Parameters:
      t : 1d array_like object integer (-1/+1)
        target values
      p : 1d array_like object integer (-1/+1)
        predicted values

    :Returns:
      sensitivity : float, in range [0.0, 1.0]
    """

    tarr = np.asarray(t, dtype=np.int)
    parr = np.asarray(p, dtype=np.int)

    if tarr.shape[0] != parr.shape[0]:
        raise ValueError("t, p: shape mismatch")
    
    if not is_binary(tarr):
        raise ValueError("only binary classification problems"
            " with t[i] = -1/+1 are allowed.")

    tp = true_pos(tarr, parr)
    ap = float((tp + false_neg(tarr, parr)))
    
    if ap == 0:
        return 0.0

    return tp / ap


def specificity(t, p):
    """Specificity, computed as:

    specificity = tn / an
    
    Only binary classification problems with t[i] = -1/+1
    are allowed.

    :Parameters:
      t : 1d array_like object integer (-1/+1)
        target values
      p : 1d array_like object integer (-1/+1)
        predicted values

    :Returns:
      sensitivity : float, in range [0.0, 1.0]
    """

    tarr = np.asarray(t, dtype=np.int)
    parr = np.asarray(p, dtype=np.int)

    if tarr.shape[0] != parr.shape[0]:
        raise ValueError("t, p: shape mismatch")
    
    if not is_binary(tarr):
        raise ValueError("only binary classification problems"
            " with t[i] = -1/+1 are allowed.")
    
    tn = true_neg(tarr, parr)
    an = float((false_pos(tarr, parr) + tn))

    if an == 0:
        return 0.0

    return tn / an


def ppv(t, p):
    """Positive Predictive Value (PPV) computed as:

    ppv = tp / pp

    Only binary classification problems with t[i] = -1/+1
    are allowed.
    
    :Parameters:
      t : 1d array_like object integer (-1/+1)
        target values
      p : 1d array_like object integer (-1/+1)
        predicted values

    :Returns:
      PPV : float, in range [0.0, 1.0]
    """

    tarr = np.asarray(t, dtype=np.int)
    parr = np.asarray(p, dtype=np.int)

    if tarr.shape[0] != parr.shape[0]:
        raise ValueError("t, p: shape mismatch")
    
    if not is_binary(tarr):
        raise ValueError("only binary classification problems"
            " with t[i] = -1/+1 are allowed.")

    tp = true_pos(tarr, parr)
    pp = float((tp + false_pos(tarr, parr)))
    
    if pp == 0:
        return 0.0

    return tp / pp


def npv(t, p):
    """Negative Predictive Value (NPV), computed as:

    npv = tn / pn

    Only binary classification problems with t[i] = -1/+1
    are allowed.
    
    :Parameters:
      t : 1d array_like object integer (-1/+1)
        target values
      p : 1d array_like object integer (-1/+1)
        predicted values

    :Returns:
      NPV : float, in range [0.0, 1.0]
    """

    tarr = np.asarray(t, dtype=np.int)
    parr = np.asarray(p, dtype=np.int)

    if tarr.shape[0] != parr.shape[0]:
        raise ValueError("t, p: shape mismatch")
    
    if not is_binary(tarr):
        raise ValueError("only binary classification problems"
            " with t[i] = -1/+1 are allowed.")
       
    tn = true_neg(tarr, parr)
    pn = float((tn + false_neg(tarr, parr)))
    
    if pn == 0:
        return 0.0  

    return tn / pn


def mcc(t, p):
    """Matthews Correlation Coefficient (MCC), computed as:

    MCC = ((tp*tn)-(fp*fn)) / sqrt((tp+fn)*(tp+fp)*(tn+fn)*(tn+fp))
    
    Only binary classification problems with t[i] = -1/+1 are allowed.
    
    Returns a value between -1 and +1. A MCC of +1 represents 
    a perfect prediction, 0 an average random prediction and 
    -1 an inverse prediction.
    If any of the four sums in the denominator is zero,
    the denominator is set to one; this results in a Matthews
    Correlation Coefficient of zero, which can be shown to be
    the correct limiting value.

    :Parameters:
      t : 1d array_like object integer (-1/+1)
        target values
      p : 1d array_like object integer (-1/+1)
        predicted values

    :Returns:
      MCC : float, in range [-1.0, 1.0]
    """

    tarr = np.asarray(t, dtype=np.int)
    parr = np.asarray(p, dtype=np.int)

    if tarr.shape[0] != parr.shape[0]:
        raise ValueError("t, p: shape mismatch")
    
    if not is_binary(tarr):
        raise ValueError("only binary classification problems"
            " with t[i] = -1/+1 are allowed.")

    tp = true_pos(tarr, parr)
    tn = true_neg(tarr, parr)
    fp = false_pos(tarr, parr)  
    fn = false_neg(tarr, parr)

    den = np.sqrt((tp+fn)*(tp+fp)*(tn+fn)*(tn+fp))
    if den == 0.0:
        den = 1.0

    num = np.float((tp*tn)-(fp*fn))
    return num / den


def auc_wmw(t, p):
    """Compute the AUC by using the Wilcoxon-Mann-Whitney
    statistic. Only binary classification problems with
    t[i] = -1/+1 are allowed.

    :Parameters:
      t : 1d array_like object integer (-1/+1)
        target values
      p : 1d array_like object (negative/positive values)
        predicted values
      
    :Returns:
      AUC : float, in range [0.0, 1.0]
    """

    tarr = np.asarray(t, dtype=np.int)
    parr = np.asarray(p, dtype=np.float)

    if tarr.shape[0] != parr.shape[0]:
        raise ValueError("t, p: shape mismatch")
    
    if not is_binary(tarr):
        raise ValueError("only binary classification problems"
            " with t[i] = -1/+1 are allowed.")
    
    idxp = np.where(tarr ==  1)[0]
    idxn = np.where(tarr == -1)[0]
    
    auc = 0.0
    for i in idxp:
        for j in idxn:
            if (p[i] - p[j]) > 0.0:
                auc += 1.0
                
    return auc / float(idxp.shape[0] * idxn.shape[0])


def mse(t, p):
    """Mean Squared Error (MSE).

    :Parameters:
      t : 1d array_like object
        target values
      p : 1d array_like object
        predicted values
      
    :Returns:
      MSE : float
    """

    tarr = np.asarray(t, dtype=np.float)
    parr = np.asarray(p, dtype=np.float)

    if tarr.shape[0] != parr.shape[0]:
        raise ValueError("t, p: shape mismatch")

    n = tarr.shape[0]

    return np.sum((tarr - parr)**2) / n


def r2(t, p):
    """Coefficient of determination (R^2)
    computed as 1 - (sserr/sstot), where `sserr` is
    the sum of squares of residuals and `sstot` is
    the total sum of squares.
    
    :Parameters:
      t : 1d array_like object
        target values
      p : 1d array_like object
        predicted values
      
    :Returns:
      R^2 : float
    """

    tarr = np.asarray(t, dtype=np.float)
    parr = np.asarray(p, dtype=np.float)

    if tarr.shape[0] != parr.shape[0]:
        raise ValueError("t, p: shape mismatch")

    sserr = np.sum((tarr - parr)**2)
    sstot = np.sum((tarr - tarr.mean())**2)

    return 1. - (sserr / sstot)


def r2_corr(t, p):
    """Coefficient of determination (R^2)
    computed as square of the correlation
    coefficient.
    
    :Parameters:
      t : 1d array_like object
        target values
      p : 1d array_like object
        predicted values
      
    :Returns:
      R^2 : float
    """

    tarr = np.asarray(t, dtype=np.float)
    parr = np.asarray(p, dtype=np.float)

    if tarr.shape[0] != parr.shape[0]:
        raise ValueError("t, p: shape mismatch")

    return np.corrcoef(parr, tarr)[0,1]**2



## This code is written by Davide Albanese, <albanese@fbk.eu>.
## (C) 2011 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np

__all__ = ["Golub"]


class Golub:
    """Golub binary classifier described in [Golub99]_.

    Decision function is D(x) = w (x-mu), where w is defined
    as w_i = (mu_i(+) - mu_i(-)) / (std_i(+) + std_i(-)) and 
    mu id defined as (mu(+) + mu(-)) / 2.

    .. [Golub99] T R Golub et al. Molecular classification of cancer: Class discovery and class prediction by gene expression monitoring. Science, 1999.
    """
    
    def __init__(self):
        """Initialization.
        """
        
        self._labels = None
        self._w = None
        self._mean = None
        
    def learn(self, x, y):
        """Learning method.
        
        :Parameters:
           x : 2d array_like object
              training data (N, P)
           y : 1d array_like object integer (only two classes)
              target values (N)
        """
        
        xarr = np.asarray(x, dtype=np.float)
        yarr = np.asarray(y, dtype=np.int)
        
        if xarr.ndim != 2:
            raise ValueError("x must be a 2d array_like object")
        
        if yarr.ndim != 1:
            raise ValueError("y must be an 1d array_like object")
        
        if xarr.shape[0] != yarr.shape[0]:
            raise ValueError("x, y: shape mismatch")
        
        self._labels = np.unique(yarr)
        k = self._labels.shape[0]

        if k != 2:
            raise ValueError("number of classes must be = 2")
        
        idxn = yarr == self._labels[0]
        idxp = yarr == self._labels[1]
        meann = np.mean(xarr[idxn], axis=0)
        meanp = np.mean(xarr[idxp], axis=0)
        stdn = np.std(xarr[idxn], axis=0, ddof=1)
        stdp = np.std(xarr[idxp], axis=0, ddof=1)
        self._w = (meanp - meann) / (stdp + stdn)
        self._mean = 0.5 * (meanp + meann)
        
    def pred(self, t):
        """Prediction method.
        
        :Parameters:
           t : 1d or 2d array_like object
              testing data ([M,], P)
        """

        if self._w is None:
            raise ValueError("no model computed")
        
        tarr = np.asarray(t, dtype=np.float)
        if tarr.ndim > 2:
            raise ValueError("t must be an 1d or a 2d array_like object")
        
        try:
            tmp = np.dot(tarr-self._mean, self._w)
        except ValueError:
            raise ValueError("t, model: shape mismatch")
        
        return np.where(tmp>0, self._labels[1], self._labels[0]) \
            .astype(np.int)
    
    def w(self):
        """Returns the coefficients.
        """
        
        if self._w is None:
            raise ValueError("no model computed")
        
        return self._w

    def labels(self):
        """Outputs the name of labels.
        """
        
        return self._labels



## This code is written by Davide Albanese, <albanese@fbk.eu>.
## (C) 2010 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ["ols_base", "OLS"]

import numpy as np

def ols_base(x, y, tol):
    """Ordinary (Linear) Least Squares.

    Solves the equation X beta = y by computing a vector beta that
    minimize ||y - X beta||^2 where ||.|| is the L^2 norm
    This function uses numpy.linalg.lstsq().

    X must be centered by columns.

    :Parameters:
       x : 2d array_like object
          training data (samples x features)
       y : 1d array_like object integer (two classes)
          target values
       tol : float
          Cut-off ratio for small singular values of x.
          Singular values are set to zero if they are smaller
          than `tol` times the largest singular value of x.
          If `tol` < 0, machine precision is used instead.

    :Returns:
       beta, rank = 1d numpy array, float
          beta, rank of matrix `x`.
    """

    beta, _, rank, _ = np.linalg.lstsq(x, y, rcond=tol)
    return beta, rank


class OLS:
    """Ordinary (Linear) Least Squares Regression (OLS).
    """

    def __init__(self, tol=-1):
        """Initialization.

        :Parameters:
           tol : float
              Cut-off ratio for small singular values of x.
              Singular values are set to zero if they are smaller
              than `tol` times the largest singular value of x.
              If `tol` < 0, machine precision is used instead.
        """

        self._beta = None
        self._beta0 = None
        self._rank = None
        self._tol = tol

    def learn(self, x, y):
        """Learning method.

        :Parameters:
           x : 2d array_like object
              training data (samples x features)
           y : 1d array_like object integer (two classes)
              target values
        """

        xarr = np.array(x, dtype=np.float, copy=True)
        yarr = np.asarray(y, dtype=np.float)

        if xarr.ndim != 2:
            raise ValueError("x must be a 2d array_like object")

        if yarr.ndim != 1:
            raise ValueError("y must be an 1d array_like object")

        if xarr.shape[0] != yarr.shape[0]:
            raise ValueError("x, y shape mismatch")

        xarr = np.concatenate((np.ones((xarr.shape[0], 1),
                   dtype=np.float), xarr), axis=1)

        beta, self._rank = ols_base(xarr, yarr, self._tol)
        self._beta = beta[1:]
        self._beta0 = beta[0]

    def pred(self, t):
        """Compute the predicted response.

        :Parameters:
           t : 1d or 2d array_like object
              test data

        :Returns:
           p : integer or 1d numpy darray
              predicted response
        """
        if not self._beta or not self._beta0:
            raise ValueError('no mode computed; run learn() first')

        tarr = np.asarray(t, dtype=np.float)

        if tarr.ndim > 2 or tarr.ndim < 1:
            raise ValueError("t must be an 1d or a 2d array_like object")

        try:
            p = np.dot(tarr, self._beta) + self._beta0
        except ValueError:
            raise ValueError("t, beta: shape mismatch")

        return p

    def beta(self):
        """Return b1, ..., bp.
        """

        return self._beta

    def beta0(self):
        """Return b0.
        """

        return self._beta0

    def rank(self):
        """Rank of matrix `x`.
        """

        return self._rank




## This code is written by Davide Albanese, <albanese@fbk.eu>
## (C) 2011 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.


import numpy as np
cimport numpy as np
from libc.stdlib cimport *

from clibml cimport *

np.import_array()


cdef class KNN:
    """k-Nearest Neighbor (euclidean distance).
    """
    cdef NearestNeighbor nn
    cdef int k
    cdef int *classes

    def __cinit__(self, k):
        """Initialization.
        
        :Parameters:
           k : int
              number of nearest neighbors
        """

        self.nn.x = NULL
        self.nn.y = NULL
        self.nn.classes = NULL
        self.k = int(k)
        self.classes = NULL
        
    def learn(self, x, y):
        """Learn method.
        
        :Parameters:	
           x : 2d array_like object (N,P)
              training data 
           y : 1d array_like integer 
              class labels
        """

        cdef int ret
        cdef np.ndarray[np.float_t, ndim=2] xarr
        cdef np.ndarray[np.int_t, ndim=1] yarr
        cdef np.ndarray[np.int32_t, ndim=1] ynew
        cdef double *xp
        cdef double **xpp
        cdef int i

        xarr = np.ascontiguousarray(x, dtype=np.float)
        yarr = np.ascontiguousarray(y, dtype=np.int)
        
        if self.k > xarr.shape[0]:
            raise ValueError("k must be smaller than number of samples")

        yu = np.unique(yarr)
        if yu.shape[0] <= 1:
            raise ValueError("y: number of classes must be >=2")

        self._free()
        
        # save original labels
        self.classes = <int *> malloc (yu.shape[0] * sizeof(int))
        for i in range(yu.shape[0]):
            self.classes[i] = yu[i]
        
        # transform labels
        ynew = np.empty(yarr.shape[0], dtype=np.int32)
        if yu.shape[0] == 2:
            cond_neg = (yarr == yu[0])
            cond_pos = (yarr == yu[1])
            ynew[cond_neg], ynew[cond_pos] = -1, 1
        else:
            for i in range(yu.shape[0]):
                cond = (yarr == yu[i])
                ynew[cond] = i + 1
    
        xp = <double *> xarr.data
        xpp = <double **> malloc (xarr.shape[0] * sizeof(double*))
        for i in range(xarr.shape[0]):
            xpp[i] = xp + (i * xarr.shape[1])
        
        ret = compute_nn(&self.nn, <int> xarr.shape[0], <int> xarr.shape[1],
                          xpp, <int *> ynew.data, self.k, DIST_EUCLIDEAN)
        free(xpp)

        if ret == 1:
            raise MemoryError("out of memory")
        
    def pred(self, t):
        """Predict KNN model on a test point(s).
        
        :Parameters:
           t : 1d or 2d array_like object ([M,] P)
              test point(s)
              
        :Returns:
	   p : int or 1d numpy array
              the predicted value(s). Retuns the smallest label
              minus one (KNN.labels()[0]-1) when the classification
              is not unique.
        """
        
        cdef int i
        cdef np.ndarray[np.float_t, ndim=1] tiarr
        cdef double *margin
        cdef double *tdata
        

        if self.nn.x is NULL:
            raise ValueError("no model computed")
        
        tarr = np.ascontiguousarray(t, dtype=np.float)
        if tarr.ndim > 2:
            raise ValueError("t must be an 1d or a 2d array_like object")

        if tarr.shape[-1] != self.nn.d:
            raise ValueError("t, model: shape mismatch")

        if tarr.ndim == 1:
            tiarr = tarr
            p = predict_nn(&self.nn, <double *> tiarr.data, &margin)
            free(margin)
            if p == -2:
                raise MemoryError("out of memory")
            elif p == 0:
                ret = self.classes[0] - 1
            else:
                if self.nn.nclasses == 2:
                    if p == -1: ret = self.classes[0]
                    else: ret = self.classes[1]
                else:
                    ret = self.classes[p-1]

        else:
            ret = np.empty(tarr.shape[0], dtype=np.int)
            for i in range(tarr.shape[0]):
                tiarr = tarr[i]
                tdata = <double *> tiarr.data
                p = predict_nn(&self.nn, tdata, &margin)
                free(margin)
                if p == -2:
                    raise MemoryError("out of memory")
                elif p == 0:
                    ret[i] = self.classes[0] - 1
                else:
                    if self.nn.nclasses == 2:
                        if p == -1: ret[i] = self.classes[0]
                        else: ret[i] = self.classes[1]
                    else:
                        ret[i] = self.classes[p-1]
            
        return ret
            
    def nclasses(self):
        """Returns the number of classes.
        """
        
        if self.nn.x is NULL:
            raise ValueError("no model computed")

        return self.nn.nclasses

    def labels(self):
        """Outputs the name of labels.
        """
        
        if self.nn.x is NULL:
            raise ValueError("no model computed")
        
        ret = np.empty(self.nn.nclasses, dtype=np.int)
        for i in range(self.nn.nclasses):
            ret[i] = self.classes[i]

        return ret

    def _free(self):
        if self.nn.x is not NULL:
            for i in range(self.nn.n):
                free(self.nn.x[i])
            free(self.nn.x)

        if self.nn.y is not NULL:
            free(self.nn.y)
        
        if self.nn.classes is not NULL:
            free(self.nn.classes)

        if self.classes is not NULL:
            free(self.classes)

    def __dealloc__(self):
        self._free()


cdef class ClassTree:
    """Classification Tree (gini index).
    """
    cdef Tree tree
    cdef int stumps
    cdef int minsize
    cdef int *classes
 
    def __cinit__(self, stumps=False, minsize=0):
        """Initialization.
        
        :Parameters:
           stumps : bool
              True: compute single split or False: standard tree
           minsize : int (>=0)
              minimum number of cases required to split a leaf
        """

        self.tree.x = NULL
        self.tree.y = NULL
        self.tree.classes = NULL
        self.tree.node = NULL
        self.classes = NULL

        self.stumps = int(bool(stumps))
        self.minsize = int(minsize)

        if self.minsize < 0:
            raise ValueError("minsize must be >= 0")
        
    def learn(self, x, y):
        """Learn method.
        
        :Parameters:	
           x : 2d array_like object (N x P)
              training data 
           y : 1d array_like integer 
              class labels
        """

        cdef int ret
        cdef np.ndarray[np.float_t, ndim=2] xarr
        cdef np.ndarray[np.int_t, ndim=1] yarr
        cdef np.ndarray[np.int32_t, ndim=1] ynew
        cdef double *xp
        cdef double **xpp
        cdef int i

        xarr = np.ascontiguousarray(x, dtype=np.float)
        yarr = np.ascontiguousarray(y, dtype=np.int)
       
        yu = np.unique(yarr)
        if yu.shape[0] <= 1:
            raise ValueError("y: number of classes must be >=2")
        
        self._free()

        # save original labels
        self.classes = <int *> malloc (yu.shape[0] * sizeof(int))
        for i in range(yu.shape[0]):
            self.classes[i] = yu[i]
            
        # transform labels
        ynew = np.empty(yarr.shape[0], dtype=np.int32)
        if yu.shape[0] == 2:
            cond_neg = (yarr == yu[0])
            cond_pos = (yarr == yu[1])
            ynew[cond_neg], ynew[cond_pos] = -1, 1
        else:
            for i in range(yu.shape[0]):
                cond = (yarr == yu[i])
                ynew[cond] = i + 1

        xp = <double *> xarr.data
        xpp = <double **> malloc (xarr.shape[0] * sizeof(double*))
        for i in range(xarr.shape[0]):
            xpp[i] = xp + (i * xarr.shape[1])
        
        ret = compute_tree(&self.tree, <int> xarr.shape[0], <int> xarr.shape[1],
                          xpp, <int *> ynew.data, self.stumps, self.minsize)
        free(xpp)

        if ret == 1:
            raise MemoryError("out of memory")
        
    def pred(self, t):
        """Predict Tree model on a test point(s).
        
        :Parameters:
           t : 1d or 2d array_like object ([M,] P)
              test point(s)
              
        :Returns:
	   p : int or 1d numpy array
              the predicted value(s). Retuns the smallest label
              minus one (ClassTree.labels()[0]-1) when the classification
              is not unique.          
        """ 
        
        cdef int i, p
        cdef np.ndarray[np.float_t, ndim=1] tiarr
        cdef double *margin
        cdef double *tdata
        

        if self.tree.x is NULL:
            raise ValueError("no model computed")
        
        tarr = np.ascontiguousarray(t, dtype=np.float)
        if tarr.ndim > 2:
            raise ValueError("t must be an 1d or a 2d array_like object")

        if tarr.shape[-1] != self.tree.d:
            raise ValueError("t, model: shape mismatch")

        if tarr.ndim == 1:
            tiarr = tarr
            p = predict_tree(&self.tree, <double *> tiarr.data, &margin)
            free(margin)
            if p == -2:
                raise MemoryError("out of memory")
            elif p == 0:
                ret = self.classes[0] - 1
            else:
                if self.tree.nclasses == 2:
                    if p == -1: ret = self.classes[0]
                    else: ret = self.classes[1]
                else:
                    ret = self.classes[p-1]

        else:
            ret = np.empty(tarr.shape[0], dtype=np.int)
            for i in range(tarr.shape[0]):
                tiarr = tarr[i]
                tdata = <double *> tiarr.data
                p = predict_tree(&self.tree, tdata, &margin)
                free(margin)
                if p == -2:
                    raise MemoryError("out of memory")
                elif p == 0:
                    ret[i] = self.classes[0] - 1
                else:
                    if self.tree.nclasses == 2:
                        if p == -1: ret[i] = self.classes[0]
                        else: ret[i] = self.classes[1]
                    else:
                        ret[i] = self.classes[p-1]
            
        return ret

    def nclasses(self):
        """Returns the number of classes.
        """
        
        if self.tree.x is NULL:
            raise ValueError("no model computed")

        return self.tree.nclasses

    def labels(self):
        """Outputs the name of labels.
        """
        
        if self.tree.x is NULL:
            raise ValueError("no model computed")
        
        ret = np.empty(self.tree.nclasses, dtype=np.int)
        for i in range(self.tree.nclasses):
            ret[i] = self.classes[i]

        return ret

    def _free(self):
        cdef int i
        
        if self.tree.x is not NULL:
            for i in range(self.tree.n):
                free(self.tree.x[i])
            free(self.tree.x)

        if self.tree.y is not NULL:
            free(self.tree.y)
        
        if self.tree.classes is not NULL:
            free(self.tree.classes)
            
        if self.tree.node is not NULL:
            free(self.tree.node[0].npoints_for_class)
            free(self.tree.node[0].priors)

            for i in range(1, self.tree.nnodes):
                free(self.tree.node[i].data)
                free(self.tree.node[i].classes)
                free(self.tree.node[i].npoints_for_class)
                free(self.tree.node[i].priors)
            
            free(self.tree.node)
    
        if self.classes is not NULL:
            free(self.classes)

    def __dealloc__(self):
        self._free()


cdef class MaximumLikelihoodC:
    """Maximum Likelihood Classifier.
    """

    cdef MaximumLikelihood ml
    cdef int *classes

    def __cinit__(self):
        """Initialization.
        """
        
        self.ml.classes = NULL
        self.ml.npoints_for_class = NULL
        self.ml.mean = NULL
        self.ml.covar = NULL
        self.ml.inv_covar = NULL
        self.ml.priors = NULL
        self.ml.det = NULL
        self.classes = NULL
        
    def learn(self, x, y):
        """Learn method.
        
        :Parameters:	
           x : 2d array_like object (N,P)
              training data 
           y : 1d array_like integer 
              class labels
        """

        cdef int ret
        cdef np.ndarray[np.float_t, ndim=2] xarr
        cdef np.ndarray[np.int_t, ndim=1] yarr
        cdef np.ndarray[np.int32_t, ndim=1] ynew
        cdef double *xp
        cdef double **xpp
        cdef int i

        xarr = np.ascontiguousarray(x, dtype=np.float)
        yarr = np.ascontiguousarray(y, dtype=np.int)

        yu = np.unique(yarr)
        if yu.shape[0] <= 1:
            raise ValueError("y: number of classes must be >=2")

        self._free()
        
        # save original labels
        self.classes = <int *> malloc (yu.shape[0] * sizeof(int))
        for i in range(yu.shape[0]):
            self.classes[i] = yu[i]
        
        # transform labels
        ynew = np.empty(yarr.shape[0], dtype=np.int32)
        if yu.shape[0] == 2:
            cond_neg = (yarr == yu[0])
            cond_pos = (yarr == yu[1])
            ynew[cond_neg], ynew[cond_pos] = -1, 1
        else:
            for i in range(yu.shape[0]):
                cond = (yarr == yu[i])
                ynew[cond] = i + 1
    
        xp = <double *> xarr.data
        xpp = <double **> malloc (xarr.shape[0] * sizeof(double*))
        for i in range(xarr.shape[0]):
            xpp[i] = xp + (i * xarr.shape[1])
        
        ret = compute_ml(&self.ml, <int> xarr.shape[0], <int> xarr.shape[1],
                          xpp, <int *> ynew.data)
        free(xpp)

        if ret == 1:
            raise MemoryError("out of memory")
        
    def pred(self, t):
        """Predict Maximum Likelihood model on a test point(s).
        
        :Parameters:
           t : 1d or 2d array_like object ([M,] P)
              test point(s)
              
        :Returns:
	   p : int or 1d numpy array
              the predicted value(s). Retuns the smallest label
              minus one (MaximumLikelihoodC.labels()[0]-1) when 
              the classification is not unique.
        """
        
        cdef int i
        cdef np.ndarray[np.float_t, ndim=1] tiarr
        cdef double *margin
        cdef double *tdata
        

        if self.ml.mean is NULL:
            raise ValueError("no model computed")
        
        tarr = np.ascontiguousarray(t, dtype=np.float)
        if tarr.ndim > 2:
            raise ValueError("t must be an 1d or a 2d array_like object")

        if tarr.shape[-1] != self.ml.d:
            raise ValueError("t, model: shape mismatch")

        if tarr.ndim == 1:
            tiarr = tarr
            p = predict_ml(&self.ml, <double *> tiarr.data, &margin)
            free(margin)
            if p == -2:
                raise MemoryError("out of memory")
            elif p == 0:
                ret = self.classes[0] - 1
            else:
                if self.ml.nclasses == 2:
                    if p == -1: ret = self.classes[0]
                    else: ret = self.classes[1]
                else:
                    ret = self.classes[p-1]

        else:
            ret = np.empty(tarr.shape[0], dtype=np.int)
            for i in range(tarr.shape[0]):
                tiarr = tarr[i]
                tdata = <double *> tiarr.data
                p = predict_ml(&self.ml, tdata, &margin)
                free(margin)
                if p == -2:
                    raise MemoryError("out of memory")
                elif p == 0:
                    ret[i] = self.classes[0] - 1
                else:
                    if self.ml.nclasses == 2:
                        if p == -1: ret[i] = self.classes[0]
                        else: ret[i] = self.classes[1]
                    else:
                        ret[i] = self.classes[p-1]
            
        return ret
            
    def nclasses(self):
        """Returns the number of classes.
        """
        
        if self.ml.mean is NULL:
            raise ValueError("no model computed")

        return self.ml.nclasses

    def labels(self):
        """Outputs the name of labels.
        """
        
        if self.ml.mean is NULL:
            raise ValueError("no model computed")
        
        ret = np.empty(self.ml.nclasses, dtype=np.int)
        for i in range(self.ml.nclasses):
            ret[i] = self.classes[i]

        return ret

    def _free(self):
        cdef int i
        cdef int j
                
        if self.ml.classes is not NULL:
            free(self.ml.classes)
        
        if self.ml.npoints_for_class is not NULL:
            free(self.ml.npoints_for_class)
        
        if self.ml.mean is not NULL:
            for i in range(self.ml.nclasses):
                free(self.ml.mean[i])
            free(self.ml.mean)
            
        if self.ml.covar is not NULL:
            for i in range(self.ml.nclasses):
                for j in range(self.ml.d):
                    free(self.ml.covar[i][j])
                free(self.ml.covar[i])
            free(self.ml.covar)
            
        if self.ml.inv_covar is not NULL:
            for i in range(self.ml.nclasses):
                for j in range(self.ml.d):
                    free(self.ml.inv_covar[i][j])
                free(self.ml.inv_covar[i])
            free(self.ml.inv_covar)

        if self.ml.priors is not NULL:
            free(self.ml.priors)
            
        if self.ml.det is not NULL:
            free(self.ml.det)
            
        if self.classes is not NULL:
            free(self.classes)

    def __dealloc__(self):
        self._free()



## This code is written by Davide Albanese, <albanese@fbk.eu>.
## (C) 2011 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np
cimport numpy as np
cimport cython

cdef extern from "c_kernel.h":
    double linear(double *x, double *y, int n)
    double polynomial(double *x, double *y, int n, double gamma, double b, double d)
    double gaussian(double *x, double *y, int n, double sigma)
    double exponential(double *x, double *y, int n, double sigma)
    double sigmoid(double *x, double *y, int n, double gamma, double b)


@cython.boundscheck(False)
def kernel_linear(t, x):
    cdef np.ndarray[np.float64_t, ndim=2] t_arr
    cdef np.ndarray[np.float64_t, ndim=2] x_arr
    cdef np.ndarray[np.float64_t, ndim=2] k_arr
    cdef int i, j, nt, nx, pt, px
    
    t_arr = np.array(t, dtype=np.float, order='C', ndmin=2)
    x_arr = np.array(x, dtype=np.float, order='C', ndmin=2)
    nt, pt = t_arr.shape[0], t_arr.shape[1]
    nx, px = x_arr.shape[0], x_arr.shape[1]

    if pt != px:
        raise ValueError("t, x: shape mismatch")
        
    k_arr = np.empty((nt, nx), dtype=np.float)
    for i in range(nt):
        for j in range(nx):
            k_arr[i, j] = linear(<double *> t_arr.data + (i * pt),
                <double *> x_arr.data + (j * px), pt)
    
    if nt == 1:
        return k_arr[0]
    else:
        return k_arr


@cython.boundscheck(False)
def kernel_polynomial(t, x, gamma=1.0, b=1.0, d=2.0):
    cdef np.ndarray[np.float64_t, ndim=2] t_arr
    cdef np.ndarray[np.float64_t, ndim=2] x_arr
    cdef np.ndarray[np.float64_t, ndim=2] k_arr
    cdef int i, j, nt, nx, pt, px
    
    t_arr = np.array(t, dtype=np.float, order='C', ndmin=2)
    x_arr = np.array(x, dtype=np.float, order='C', ndmin=2)
    nt, pt = t_arr.shape[0], t_arr.shape[1]
    nx, px = x_arr.shape[0], x_arr.shape[1]

    if pt != px:
        raise ValueError("t, x: shape mismatch")
        
    k_arr = np.empty((nt, nx), dtype=np.float)
    for i in range(nt):
        for j in range(nx):
            k_arr[i, j] = polynomial(<double *> t_arr.data + (i * pt),
                <double *> x_arr.data + (j * px), pt, gamma, b, d)
    
    if nt == 1:
        return k_arr[0]
    else:
        return k_arr


@cython.boundscheck(False)
def kernel_gaussian(t, x, sigma=1.0):
    cdef np.ndarray[np.float64_t, ndim=2] t_arr
    cdef np.ndarray[np.float64_t, ndim=2] x_arr
    cdef np.ndarray[np.float64_t, ndim=2] k_arr
    cdef int i, j, nt, nx, pt, px
    
    t_arr = np.array(t, dtype=np.float, order='C', ndmin=2)
    x_arr = np.array(x, dtype=np.float, order='C', ndmin=2)
    nt, pt = t_arr.shape[0], t_arr.shape[1]
    nx, px = x_arr.shape[0], x_arr.shape[1]

    if pt != px:
        raise ValueError("t, x: shape mismatch")
        
    k_arr = np.empty((nt, nx), dtype=np.float)
    for i in range(nt):
        for j in range(nx):
            k_arr[i, j] = gaussian(<double *> t_arr.data + (i * pt),
                <double *> x_arr.data + (j * px), pt, sigma)

    if nt == 1:
        return k_arr[0]
    else:
        return k_arr
    

@cython.boundscheck(False)
def kernel_exponential(t, x, sigma=1.0):
    cdef np.ndarray[np.float64_t, ndim=2] t_arr
    cdef np.ndarray[np.float64_t, ndim=2] x_arr
    cdef np.ndarray[np.float64_t, ndim=2] k_arr
    cdef int i, j, nt, nx, pt, px
    
    t_arr = np.array(t, dtype=np.float, order='C', ndmin=2)
    x_arr = np.array(x, dtxpe=np.float, order='C', ndmin=2)
    nt, pt = t_arr.shape[0], t_arr.shape[1]
    nx, px = x_arr.shape[0], x_arr.shape[1]

    if pt != px:
        raise ValueError("t, x: shape mismatch")
        
    k_arr = np.empty((nt, nx), dtype=np.float)
    for i in range(nt):
        for j in range(nx):
            k_arr[i, j] = exponential(<double *> t_arr.data + (i * pt),
                <double *> x_arr.data + (j * px), pt, sigma)
    
    if nt == 1:
        return k_arr[0]
    else:
        return k_arr


@cython.boundscheck(False)
def kernel_sigmoid(t, x, gamma=1.0, b=1.0):
    cdef np.ndarray[np.float64_t, ndim=2] t_arr
    cdef np.ndarray[np.float64_t, ndim=2] x_arr
    cdef np.ndarray[np.float64_t, ndim=2] k_arr
    cdef int i, j, nt, nx, pt, px
    
    t_arr = np.array(t, dtype=np.float, order='C', ndmin=2)
    x_arr = np.array(x, dtype=np.float, order='C', ndmin=2)
    nt, pt = t_arr.shape[0], t_arr.shape[1]
    nx, px = x_arr.shape[0], x_arr.shape[1]

    if pt != px:
        raise ValueError("t, x: shape mismatch")
        
    k_arr = np.empty((nt, nx), dtype=np.float)
    for i in range(nt):
        for j in range(nx):
            k_arr[i, j] = sigmoid(<double *> t_arr.data + (i * pt),
                <double *> x_arr.data + (j * px), pt, gamma, b)
    
    if nt == 1:
        return k_arr[0]
    else:
        return k_arr
   

def kernel_center(Kt, K):
    """Kernel matrix centering.
    """
    
    Kt_arr = np.array(Kt, dtype=np.float)
    K_arr = np.array(K, dtype=np.float)

    if Kt_arr.ndim == 1:
        J1 = np.mean(Kt_arr)
    else:
        J1 = np.mean(Kt_arr, axis=1).reshape(-1, 1)

    J2 = np.mean(K_arr, axis=0)

    return Kt_arr - J1 - J2 + np.mean(K_arr)





## Iterative RELIEF for Feature Weighting.

## This is an implementation of Iterative RELIEF algorithm described in:
## Yijun Sun. 'Iterative RELIEF for Feature Weightinig: Algorithms,
## Theories and Application'. In IEEE Transactions on Pattern Analysis
## and Machine Intelligence, 2006.
    
## This code is written by Davide Albanese, <albanese@fbk.eu>.
## (C) 2007 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ['SigmaError', 'IRelief']

from numpy import *


class SigmaError(Exception):
    pass


def norm_w(x, w):
    """
    Compute sum_i( w[i] * |x[i]| ).

    See p. 7.
    """
    return (w * abs(x)).sum()


def norm(x, n):
    """
    Compute n-norm.
    """
    return (sum(abs(x)**n))**(1.0/float(n))


def kernel(d, sigma):
    """
    Kernel.

    See p. 7.
    """
    return exp(-d/float(sigma))  


def compute_M_H(y):
    """
    Compute sets M[n] = {i:1<=i<=N, y[i]!=y[n]}.
    Compute sets H[n] = {i:1<=i<=N, y[i]==y[n], i!=n}.

    See p. 6.
    """
    M, H = [], []
    for n in range(y.shape[0]):
        Mn = where(y != y[n])[0].tolist()
        M.append(Mn)
        Hn = where(y == y[n])[0]
        Hn = Hn[Hn != n].tolist()
        H.append(Hn)
    return (M, H)
    

def compute_distance_kernel(x, w, sigma):
    """
    Compute matrix dk[i][j] = f(||x[i] - x[j]||_w).

    See p. 7.
    """
    d = zeros((x.shape[0], x.shape[0]), dtype = float)
    for i in range(x.shape[0]):
        for j in range(i + 1, x.shape[0]):
            d[i][j] = norm_w(x[i]-x[j], w)
            d[j][i] = d[i][j]
    dk = kernel(d, sigma)
   
    return dk


def compute_prob(x, dist_k, i, n, indices):
    """
    See Eqs. (8), (9)
    """

    den = dist_k[n][indices].sum()    
    if den == 0.0:
        raise SigmaError("sigma (kernel parameter) too small")
    
    return dist_k[n][i] / float(den)


def compute_gn(x, dist_k, n, Mn):
    """
    See p. 7 and Eq. (10).
    """

    num = dist_k[n][Mn].sum()
    R = range(x.shape[0])
    R.remove(n)
    den = dist_k[n][R].sum()
    if den == 0.0:
        raise SigmaError("sigma (kernel parameter) too small")

    return 1.0 - (num / float(den))
       

def compute_w(x, w, M, H, sigma):
    """
    See Eq. (12).
    """

    N = x.shape[0]
    I = x.shape[1]

    # Compute ni
    ni = zeros(I, dtype = float)
    dist_k = compute_distance_kernel(x, w, sigma)
    for n in range(N):        
        m_n = zeros(I, dtype = float)
        h_n = zeros(I, dtype = float)
        for i in M[n]:
            a_in = compute_prob(x, dist_k, i, n, M[n])
            m_in = abs(x[n] - x[i])
            m_n += a_in * m_in
        for i in H[n]:
            b_in = compute_prob(x, dist_k, i, n, H[n])
            h_in = abs(x[n] - x[i])
            h_n += b_in * h_in        
        g_n = compute_gn(x, dist_k, n, M[n])
        ni += g_n * (m_n - h_n)            

    ni = ni / float(N)
        
    # Compute (ni)+ / ||(ni)+||_2
    ni_p = maximum(ni, 0.0)
    ni_p_norm2 = norm(ni_p, 2)
   
    return ni_p / ni_p_norm2


def compute_irelief(x, y, T, sigma, theta):
    """
    See I-RELIEF Algorithm at p. 8.
    """

    w_old = ones(x.shape[1]) / float(x.shape[1])
    M, H = compute_M_H(y)
    
    for t in range(T):
        w = compute_w(x, w_old, M, H, sigma) 
        stp = norm(w - w_old, 2)
        if stp < theta:
            break
        w_old = w
    return (w, t + 1)


class IRelief:
    """Iterative RELIEF for feature weighting.
    """
   
    def __init__(self, T=1000, sigma=1.0, theta=0.001):
        """
        :Parameters:
          T : integer (> 0)
            max loops
          sigma : float (> 0.0)
            kernel width
          theta : float (> 0.0)
            convergence parameter
        """

        if T <= 0:
            raise ValueError("T (max loops) must be > 0")
        if sigma <= 0.0:
            raise ValueError("sigma (kernel width) must be > 0.0")
        if theta <= 0.0:
            raise ValueError("theta (convergence parameter) must be > 0.0")
         
        self._T = T
        self._sigma = sigma
        self._theta = theta
        self._loops = None
        self._w = None

    def learn(self, x, y):
        """Compute the feature weights.

        :Parameters:
           x : 2d array_like object
              training data (N, P)
           y : 1d array_like object integer (only two classes)
              target values (N)

        :Raises:
           SigmaError
        """
        
        xarr = asarray(x, dtype=float)
        yarr = asarray(y, dtype=int)

        if xarr.ndim != 2:
            raise ValueError("x must be a 2d array_like object")
        
        if yarr.ndim != 1:
            raise ValueError("y must be an 1d array_like object")
        
        if xarr.shape[0] != yarr.shape[0]:
            raise ValueError("x, y: shape mismatch")

        if unique(y).shape[0] != 2:
            raise ValueError("number of classes must be = 2")

        self._w, self._loops = \
            compute_irelief(xarr, yarr, self._T, self._sigma, self._theta)

    def weights(self):
        """Returns the feature weights.
        """
        
        if self._w is None:
            raise ValueError("no model computed.")

        return self._w

    def loops(self):
        """Returns the number of loops.
        """

        if self._w is None:
            raise ValueError("no model computed.")

        return self._loops
        



cdef extern from "liblinear/linear.h":
    
    cdef struct feature_node:
        int index
        double value
    
    cdef struct problem:
        int l, n # l samples, n features
        int *y
        feature_node **x
        double bias
    
    cdef struct parameter:
        int solver_type

        # these are for training only
        double eps  # stopping criteria
        double C
        int nr_weight
        int *weight_label
        double* weight

    cdef struct model:
        parameter param	# parameter
        int nr_class	# number of classes
        int nr_feature
        double *w
        int *label    # label of each class
        double bias

    void set_print_string_function(void (*print_func)(char *))
        
    model* train(problem *prob, parameter *param)
    int predict(model *model_, feature_node *x)
    int predict_values(model *model_, feature_node *x, double* dec_values)
    int predict_probability(model *model_, feature_node *x, double* prob_estimates)

    int save_model(char *model_file_name, model *model_)
    model *load_model(char *model_file_name)

    void free_and_destroy_model(model **model_ptr_ptr)
    void destroy_param(parameter* param)

    char *check_parameter(problem *prob, parameter *param)



## This code is written by Davide Albanese, <albanese@fbk.eu>
## (C) 2010 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.


import numpy as np
cimport numpy as np
from libc.stdlib cimport *

from clibsvm cimport *

   
cdef void print_null(char *s):
   pass

# array 1D to svm node
cdef svm_node *array1d_to_node(np.ndarray[np.float64_t, ndim=1] x):
    cdef int i, k
    cdef np.ndarray[np.int_t, ndim=1] nz
    cdef svm_node *ret

    nz = np.nonzero(x)[0]
    ret = <svm_node*> malloc ((nz.shape[0]+1) * sizeof(svm_node))
            
    k = 0
    for i in nz:
        ret[k].index = i+1
        ret[k].value = x[i]
        k += 1
    ret[nz.shape[0]].index = -1
            
    return ret

# array 2D to svm node
cdef svm_node **array2d_to_node(np.ndarray[np.float64_t, ndim=2] x):
    cdef int i
    cdef svm_node **ret

    ret = <svm_node **> malloc \
        (x.shape[0] * sizeof(svm_node *))
    
    for i in range(x.shape[0]):
        ret[i] = array1d_to_node(x[i])
            
    return ret

# array 1D to vector
cdef double *array1d_to_vector(np.ndarray[np.float64_t, ndim=1] y):
     cdef int i
     cdef double *ret

     ret = <double *> malloc (y.shape[0] * sizeof(double))

     for i in range(y.shape[0]):
         ret[i] = y[i]

     return ret


cdef class LibSvm:
    cdef svm_problem problem
    cdef svm_parameter parameter
    cdef svm_model *model 
        
    cdef int learn_disabled

    SVM_TYPE = ['c_svc',
                'nu_svc',
                'one_class',
                'epsilon_svr',
                'nu_svr']

    KERNEL_TYPE = ['linear',
                   'poly',
                   'rbf',
                   'sigmoid']

    def __cinit__(self, svm_type='c_svc', kernel_type='linear', 
                  degree=3, gamma=0.001, coef0=0, C=1, nu=0.5,
                  eps=0.001, p=0.1, cache_size=100, shrinking=True,
                  probability=False, weight={}):
        """LibSvm.
        
        :Parameters:

            svm_type : string
                SVM type, can be one of: 'c_svc', 'nu_svc', 
                'one_class', 'epsilon_svr', 'nu_svr'. The method load_model()
                can overwrite this parameter 
            kernel_type : string
                kernel type, can be one of: 'linear' (u'*v),
                'poly' ((gamma*u'*v + coef0)^degree), 'rbf' 
                (exp(-gamma*|u-v|^2)), 'sigmoid'
                (tanh(gamma*u'*v + coef0)). The method load_model()
                can overwrite this parameter
            degree : int (for 'poly' kernel_type)
                degree in kernel. The method load_model()
                can overwrite this parameter
            gamma : float (for 'poly', 'rbf', 'sigmoid' kernel_type)
                gamma in kernel (e.g. 1 / number of features).
                The method load_model() can overwrite this parameter
            coef0 : float (for 'poly', 'sigmoid' kernel_type)
                coef0 in kernel. The method load_model()
                can overwrite this parameter
            C : float (for 'c_svc', 'epsilon_svr', 'nu_svr')
                cost of constraints violation
            nu : float (for 'nu_svc', 'one_class', 'nu_svr')
                nu parameter
            eps : float
                stopping criterion, usually 0.00001 in nu-SVC,
                0.001 in others
            p : float (for 'epsilon_svr')
                p is the epsilon in epsilon-insensitive loss function
                of epsilon-SVM regression
            cache_size : float [MB]
                size of the kernel cache, specified in megabytes
            shrinking : bool
                use the shrinking heuristics
            probability : bool
                predict probability estimates
            weight : dict 
                changes the penalty for some classes (if the weight for a
                class is not changed, it is set to 1). For example, to
                change penalty for classes 1 and 2 to 0.5 and 0.8
                respectively set weight={1:0.5, 2:0.8}
        """
        
        svm_set_print_string_function(&print_null)
        self.learn_disabled = 0

        try:
            self.parameter.svm_type = self.SVM_TYPE.index(svm_type)
        except ValueError:
            raise ValueError("invalid svm_type")
	
        try:
            self.parameter.kernel_type = self.KERNEL_TYPE.index(kernel_type)
        except ValueError:
            raise ValueError("invalid kernel_type")
	
        self.parameter.degree = degree
        self.parameter.gamma = gamma
        self.parameter.coef0 = coef0
        self.parameter.C = C
        self.parameter.nu = nu
        self.parameter.eps = eps
        self.parameter.p = p
        self.parameter.cache_size = cache_size
        self.parameter.shrinking = int(shrinking)
        self.parameter.probability = int(probability)
	
        # weight
        self.parameter.nr_weight = len(weight)
        self.parameter.weight_label = <int *> malloc \
            (len(weight) * sizeof(int))
        self.parameter.weight = <double *> malloc \
            (len(weight) * sizeof(double))
        try:
            for i, key in enumerate(weight):
                self.parameter.weight_label[i] = int(key)
                self.parameter.weight[i] = float(weight[key])
        except ValueError:
            raise ValueError("invalid weight")
        
        self.model = NULL
                    
    def __dealloc__(self):
        self._free_problem()
        self._free_model()
        self._free_param()

    def _load_problem(self, x, y):
        """Convert the data into libsvm svm_problem struct
        """

        xarr = np.ascontiguousarray(x, dtype=np.float64)
        yarr = np.ascontiguousarray(y, dtype=np.float64)
        
        if xarr.ndim != 2:
            raise ValueError("x must be a 2d array_like object")

        if yarr.ndim != 1:
            raise ValueError("y must be an 1d array_like object")

        if xarr.shape[0] != yarr.shape[0]:
            raise ValueError("x, y: shape mismatch")
        
        self.problem.x = array2d_to_node(xarr)
        self.problem.y = array1d_to_vector(yarr)
        self.problem.l = xarr.shape[0]
                
    def learn(self, x, y):
        """Constructs the model.
        For classification, y is an integer indicating the class label
        (multi-class is supported). For regression, y is the target
        value which can be any real number. For one-class SVM, it's not used
        so can be any number.
        
        :Parameters:
        
            x : 2d array_like object
                training data (N, P)
            y : 1d array_like object
                target values (N)
        """
        
        cdef char *ret

        srand(1)

        if self.learn_disabled:
            raise ValueError("learn method is disabled (model from file)")
        
        self._free_problem()
        self._load_problem(x, y)
        ret = svm_check_parameter(&self.problem, &self.parameter)
	
        if ret != NULL:
            raise ValueError(ret)

        self._free_model()       
        self.model = svm_train(&self.problem, &self.parameter)
        
    def pred(self, t):
        """Does classification or regression on test vector(s) t.
                
        :Parameters:
            t : 1d (one sample) or 2d array_like object
                test data ([M,] P)
            
        :Returns:
            p : for a classification model, the predicted class(es) for t is
                returned. For a regression model, the function value(s) of t
                calculated using the model is returned. For an one-class
                model, +1 or -1 is returned.
        """

        cdef int i
        cdef svm_node *test_node

        tarr = np.ascontiguousarray(t, dtype=np.float64)

        if tarr.ndim > 2:
            raise ValueError("t must be an 1d or a 2d array_like object")
        
        if self.model is NULL:
            raise ValueError("no model computed")

        if tarr.ndim == 1:
            test_node = array1d_to_node(tarr)
            p = svm_predict(self.model, test_node)
            free(test_node)
        else:
            p = np.empty(tarr.shape[0], dtype=np.float64)
            for i in range(tarr.shape[0]):
                test_node = array1d_to_node(tarr[i])
                p[i] = svm_predict(self.model, test_node)
                free(test_node)

        return p

    def pred_values(self, t):
        """Returns D decision values. 
        For a classification model with C classes, this method
        returns D=C*(C-1)/2 decision values for each test sample. 
        The order is label[0] vs. label[1], ..., label[0] vs. 
        label[C-1], label[1] vs. label[2], ..., label[C-2] vs. 
        label[C-1], where label can be obtained from the method labels().
        
        For a one-class model, this method returns D=1 decision value 
        for each test sample.
        
        For a regression model, this method returns the predicted
        value as in pred()
                
        :Parameters:
            t : 1d (one sample) or 2d array_like object
                test data ([M,] P)
            
        :Returns:
            decision values : 1d (D) or 2d numpy array (M,D)
                decision values for each observation.
        """

        cdef int i, j
        cdef svm_node *test_node
        cdef double *dec_values

        tarr = np.ascontiguousarray(t, dtype=np.float64)

        if tarr.ndim > 2:
            raise ValueError("t must be an 1d or a 2d array_like object")
        
        if self.model is NULL:
            raise ValueError("no model computed")

        if self.SVM_TYPE[self.parameter.svm_type] == 'c_svc' or \
                self.SVM_TYPE[self.parameter.svm_type] == 'nu_svc':
            n = self.model.nr_class*(self.model.nr_class - 1) / 2 
        else:
            n = 1

        dec_values = <double *> malloc (n * sizeof(double))
        
        if tarr.ndim == 1:
            dec_values_arr = np.empty(n, dtype=np.float)
            test_node = array1d_to_node(tarr)
            p = svm_predict_values(self.model, test_node, dec_values)
            free(test_node)
            for j in range(n):
                dec_values_arr[j] = dec_values[j]
        else:
            dec_values_arr = np.empty((tarr.shape[0], n), dtype=np.float)
            for i in range(tarr.shape[0]):
                test_node = array1d_to_node(tarr[i])
                p = svm_predict_values(self.model, test_node, dec_values)
                free(test_node)
                for j in range(n):
                    dec_values_arr[i, j] = dec_values[j]
        
        free(dec_values)
        return dec_values_arr

    def pred_probability(self, t):
        """Returns C (number of classes) probability estimates.
        For a 'c_svc' and 'nu_svc' classification models with probability 
        information, this method computes 'number of classes' probability 
        estimates.

        :Parameters:
            t : 1d (one sample) or 2d array_like object
                test data ([M,] P)
            
        :Returns:
            probability estimates : 1d (C) or 2d numpy array (M,C)
                probability estimates for each observation.
        """
        
        cdef int i, j
        cdef svm_node *test_node
        cdef double *prob_estimates

        tarr = np.ascontiguousarray(t, dtype=np.float64)

        if tarr.ndim > 2:
            raise ValueError("t must be an 1d or a 2d array_like object")
        
        if self.model is NULL:
            raise ValueError("no model computed")
        
        if self.SVM_TYPE[self.parameter.svm_type] != 'c_svc' and \
                self.SVM_TYPE[self.parameter.svm_type] != 'nu_svc':
            raise ValueError("probability estimates are available only for"
                             "'c_svc', 'nu_svc' svm types")
        
        ret = svm_check_probability_model(self.model)
        if ret == 0:
            raise ValueError("model does not contain required information"
                             " to do probability estimates. Set probability"
                             "=True")

        prob_estimates = <double*> malloc (self.model.nr_class * 
                                           sizeof(double))
        
        if tarr.ndim == 1:
            prob_estimates_arr = np.empty(self.model.nr_class, dtype=np.float)
            test_node = array1d_to_node(tarr)
            p = svm_predict_probability(self.model, test_node,
                prob_estimates)
            free(test_node)
            for j in range(self.model.nr_class):
                prob_estimates_arr[j] = prob_estimates[j]
        else:
            prob_estimates_arr = np.empty((tarr.shape[0], self.model.nr_class), 
                                          dtype=np.float)
            for i in range(tarr.shape[0]):
                test_node = array1d_to_node(tarr[i])
                p = svm_predict_probability(self.model, test_node,
                                            prob_estimates)
                free(test_node)
                for j in range(self.model.nr_class):
                    prob_estimates_arr[i, j] = prob_estimates[j]

        free(prob_estimates)
        return prob_estimates_arr
   

    def labels(self):
        """For a classification model, this method outputs the name of
        labels. For regression and one-class models, this method
        returns None.
        """
        
        if self.model is NULL:
            raise ValueError("no model computed")

        if self.model.label is NULL:
            return None
        else:
            ret = np.empty(self.model.nr_class, dtype=np.int32)
            for i in range(self.model.nr_class):
                ret[i] = self.model.label[i]
            return ret

    def nclasses(self):
        """Get the number of classes.
        = 2 in regression and in one class SVM
        """
        
        if self.model is NULL:
            raise ValueError("no model computed")

        return self.model.nr_class

    def nsv(self):
        """Get the total number of support vectors.
        """
        
        if self.model is NULL:
            raise ValueError("no model computed")

        return self.model.l

    def label_nsv(self):
        """Return a dictionary containing the number 
        of support vectors for each class (for classification).
        """

        cdef int i
        
        if self.model is NULL:
            raise ValueError("no model computed")

        nsv = {}

        if self.model.nSV is not NULL:            
            for i in range(self.model.nr_class):
                nsv[self.model.label[i]] = self.model.nSV[i]
            
        return nsv
    
    cdef void _free_problem(self):
        cdef int i
        
        if self.problem.x is not NULL:
            for i in range(self.problem.l):
                free(self.problem.x[i])
            free(self.problem.x)

        if self.problem.y is not NULL:
            free(self.problem.y)
        
    cdef void _free_model(self):
        svm_free_and_destroy_model(&self.model)

    cdef void _free_param(self):
        svm_destroy_param(&self.parameter)
    
    @classmethod
    def load_model(cls, filename):
        """Loads model from file. Returns a LibSvm object
        with the learn() method disabled.
        """
        
        ret = LibSvm()

        try:
            ret.model = svm_load_model(filename)
        except:
            raise ValueError("invalid filename")
        else:
            if ret.model is NULL:
                raise IOError("model file could not be loaded")
            
        ret.parameter.svm_type = ret.model.param.svm_type
        ret.parameter.kernel_type = ret.model.param.kernel_type
        ret.parameter.degree = ret.model.param.degree
        ret.parameter.gamma = ret.model.param.gamma
        ret.parameter.coef0 = ret.model.param.coef0
        ret.learn_disabled = 1
        
        return ret

    def save_model(self, filename):
        """Saves model to a file.
        """
        
        cdef int ret
        
        if self.model is NULL:
            raise ValueError("no model computed")
        
        ret = svm_save_model(filename, self.model)
        
        if ret == -1:
            raise IOError("problem with svm_save_model()")



## Cross Validation Submodule

## This code is written by Davide Albanese, <albanese@fbk.eu>.
## (C) 2010 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ['cv_kfold', 'cv_random', 'cv_all']


import itertools
import numpy as np


def cv_kfold(n, k, strat=None, seed=0):
    """Returns train and test indexes for k-fold 
    cross-validation.
    
    :Parameters:
       n : int (n > 1) 
          number of indexes
       k : int (k > 1) 
          number of iterations (folds). The case `k` = `n`
          is known as leave-one-out cross-validation.
       strat : None or 1d array_like integer (of length `n`)
          labels for stratification. If `strat` is not None
          returns 'stratified' k-fold CV indexes, where
          each subsample has roughly the same label proportions
          of `strat`.
       seed : int
          random seed

    :Returns:
       idx: list of tuples
          list of `k` tuples containing the train and 
          test indexes
    
    Example:

    >>> import mlpy
    >>> idx = mlpy.cv_kfold(n=12, k=3)
    >>> for tr, ts in idx: tr, ts
    ... 
    (array([2, 8, 1, 7, 9, 3, 0, 5]), array([ 6, 11,  4, 10]))
    (array([ 6, 11,  4, 10,  9,  3,  0,  5]), array([2, 8, 1, 7]))
    (array([ 6, 11,  4, 10,  2,  8,  1,  7]), array([9, 3, 0, 5]))
    >>> strat = [0,0,0,0,0,0,0,0,1,1,1,1]
    >>> idx = mlpy.cv_kfold(12, k=4, strat=strat)
    >>> for tr, ts in idx: tr, ts
    ... 
    (array([ 1,  7,  3,  0,  5,  4,  8, 10,  9]), array([ 6,  2, 11]))
    (array([ 6,  2,  3,  0,  5,  4, 11, 10,  9]), array([1, 7, 8]))
    (array([ 6,  2,  1,  7,  5,  4, 11,  8,  9]), array([ 3,  0, 10]))
    (array([ 6,  2,  1,  7,  3,  0, 11,  8, 10]), array([5, 4, 9]))
    """

    
    if n < 2:
        raise ValueError("n must be > 1")

    if k < 2:
        raise ValueError("k must be > 1")
    
    if strat is not None:
        _strat = np.asarray(strat, dtype=np.int)
        if n != _strat.shape[0]:
            raise ValueError("a, strat: shape mismatch")
    else:
        _strat = np.zeros(n, dtype=np.int)
    
    labels = np.unique(_strat)

    # check k
    kmax = np.min([np.sum(l == _strat) for l in labels])
    if k > kmax:
        raise ValueError('k must be <= %d' % kmax)

    np.random.seed(seed)

    splits = []
    for l in labels:
        tmp = np.where(l == _strat)[0]
        np.random.shuffle(tmp)
        splits.append(np.array_split(tmp, k))

    idx = []
    for i in range(k):
        idx1, idx2 = [], []
        for j in range(len(splits)):
            idx1.extend(splits[j][:i] + splits[j][i+1:])
            idx2.extend(splits[j][i])
        idx.append((np.concatenate(idx1), np.asarray(idx2)))
            
    return idx


def cv_random(n, k, p, strat=None, seed=0):
    """Returns train and test indexes for random subsampling 
    cross-validation. The proportion of the train/test indexes
    is not dependent on the number of iterations `k`.
        
    :Parameters:
       n : int (n > 1)
          number of indexes
       k : int (k > 0) 
          number of iterations (folds)
       p : float (0 <= p <= 100) 
          percentage of indexes in test
       strat : None or  1d array_like integer (of length `n`)
          labels for stratification. If `strat` is not None
          returns 'stratified' random subsampling CV indexes,
          where each subsample has roughly the same label 
          proportions of `strat`.
       seed : int
          random seed
          
    :Returns:
        idx: list of tuples
          list of `k` tuples containing the train and 
          test indexes

    Example:
    
    >>> import mlpy
    >>> ap = mlpy.cv_random(n=12, k=4, p=30)
    >>> for tr, ts in ap: tr, ts
    ... 
    (array([ 6, 11,  4, 10,  2,  8,  1,  7,  9]), array([3, 0, 5]))
    (array([ 5,  2,  3,  4,  9,  0, 11,  7,  6]), array([ 1, 10,  8]))
    (array([ 6,  1, 10,  2,  7,  5, 11,  0,  3]), array([4, 9, 8]))
    (array([2, 4, 8, 9, 5, 6, 1, 0, 7]), array([10, 11,  3]))
    """

    if n < 2:
        raise ValueError("n must be > 1")

    if k < 2:
        raise ValueError("k must be > 1")
    
    if (p < 0) or (p > 100):
        raise ValueError("p must be in [0, 100]")

    if strat is not None:
        _strat = np.asarray(strat, dtype=np.int)
        if n != _strat.shape[0]:
            raise ValueError("a, strat: shape mismatch")
    else:
        _strat = np.zeros(n, dtype=np.int)

    labels = np.unique(_strat)

    # check p
    pmin = np.min([np.sum(l == _strat) for l in labels])**-1 * 100
    if p < pmin:
        raise ValueError('p must be >= %.3f' % pmin)
            
    np.random.seed(seed)

    idx = []
    for _ in range(k):        
        idx1, idx2 = [], []
        for l in labels:
            tmp = np.where(l == _strat)[0]
            g = tmp.shape[0] - int(0.01*p*tmp.shape[0])
            np.random.shuffle(tmp)
            idx1.append(tmp[:g])
            idx2.append(tmp[g:])
        
        idx.append((np.concatenate(idx1), np.concatenate(idx2)))

    return idx


def cv_all(n, p):
    """Returns train and test indexes for all-combinations 
    cross-validation.

    :Parameters:
       n : int (n > 1)
          number of indexes
       p : float (0 <= p <= 100) 
          percentage of indexes in test

    :Returns:
       idx : list of tuples
          list of tuples containing the train and 
          test indexes

    Example

    >>> import mlpy
    >>> idx = mlpy.cv_all(n=4, p=50)
    >>> for tr, ts in idx: tr, ts
    ... 
    (array([2, 3]), array([0, 1]))
    (array([1, 3]), array([0, 2]))
    (array([1, 2]), array([0, 3]))
    (array([0, 3]), array([1, 2]))
    (array([0, 2]), array([1, 3]))
    (array([0, 1]), array([2, 3]))
    >>> idx = mlpy.cv_all(a, 10) # ValueError: p must be >= 25.000
    """

    if n < 2:
        raise ValueError("n must be > 1")

    if (p < 0.0) or (p > 100.0):
        raise ValueError("p must be in [0, 100]")
    
    # check p
    pmin = n**-1 * 100
    if p < pmin:
        raise ValueError('p must be >= %.3f' % pmin)
    
    k = int(0.01 * p * n)
    a = np.arange(n)
    tmp = np.asarray(list(itertools.combinations(a, k)))

    idx = []
    for idx2 in tmp:
        idx1 = np.setdiff1d(a, idx2)
        idx.append((idx1, idx2))

    return idx



import numpy as np
cimport numpy as np

cdef extern from "c_kmeans.h":

    void init_std(double *data, double *means, int nn, int pp, int kk, unsigned long seed)
    void init_plus(double *data, double *means, int nn, int pp, int kk, unsigned long seed)
    int km(double *data, double *means, int *cls, int nn, int pp, int kk)


def kmeans(x, k, plus=False, seed=0):
    """k-means clustering.

    :Parameters:
       x : 2d array_like object (N, P)
          data
       k : int (1<k<N)
          number of clusters
       plus : bool
          k-means++ algorithm for initialization
       seed : int
          random seed for initialization

    :Returns:
       clusters, means, steps: 1d array, 2d array, int
          cluster membership in 0,...,K-1, means (K,P), number of steps
          
    """

    cdef np.ndarray[np.float64_t, ndim=2] x_arr
    cdef np.ndarray[np.float64_t, ndim=2] means_arr
    cdef np.ndarray[np.int32_t, ndim=1] cls_arr
  
    x_arr = np.ascontiguousarray(x, dtype=np.float)

    if k <= 1 or k >= x_arr.shape[0]:
        raise ValueError("k must be in [2, N-1]")

    means_arr = np.empty((k, x_arr.shape[1]), dtype=np.float)
    cls_arr = np.empty(x_arr.shape[0], dtype=np.int32)
    
    if plus:
        init_plus(<double *> x_arr.data, <double *> means_arr.data,
                <int> x_arr.shape[0], <int> x_arr.shape[1], <int> k,
                <unsigned long> seed)
    else:
        init_std(<double *> x_arr.data, <double *> means_arr.data,
                  <int> x_arr.shape[0], <int> x_arr.shape[1], <int> k,
                  <unsigned long> seed)
        
    steps = km(<double *> x_arr.data,  <double *> means_arr.data,
                <int *> cls_arr.data, <int> x_arr.shape[0], 
                <int> x_arr.shape[1], <int> k)
                
    return cls_arr.astype(np.int), means_arr, steps



## This code is written by Davide Albanese, <albanese@fbk.eu>.
## (C) 2010 mlpy Developers.

## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ["lars_base", "LARS"]

import numpy as np


def lars_base(x, y, maxsteps=None):
    """Least Angle Regression.

    `x` should be centered and normalized by columns, and `y`
    should be centered.

     :Parameters:
        x : 2d array_like object (N x P)
           matrix of regressors
        y : 1d array_like object (N)
           response
        maxsteps : int (> 0) or None
           maximum number of steps. If `maxsteps` is None,
           the maximum number of steps is min(N-1, P),
           where N is the number of variables and P is the
           number of features.

     :Returns:
        active, est, steps : 1d numpy array, 2d numpy array, int
           active features, all LARS estimates, number of
           steps performed
    """

    xarr = np.asarray(x)
    yarr = np.asarray(y)

    ms = np.min((xarr.shape[0]-1, xarr.shape[1]))
    if (maxsteps == None) or (maxsteps > ms):
        maxsteps = ms

    mu = np.ones(xarr.shape[0])
    active = []
    inactive = range(xarr.shape[1])
    beta = np.zeros(xarr.shape[1], dtype=np.float)
    est = np.zeros((maxsteps+1, xarr.shape[1]), dtype=np.float)

    for i in range(maxsteps):

        # equation 2.8
        c = np.dot(xarr.T, (yarr - mu))

        # equation 2.9
        ct = c.copy()
        ct[active] = 0.0 # avoid re-selections
        ct_abs = np.abs(ct)
        j = np.argmax(ct_abs)

        C = ct_abs[j]
        active.append(j)
        inactive.remove(j)

        # equation 2.10
        s = np.sign(c[active])

        # equation 2.4
        xa = xarr[:, active] * s

        # equation 2.5
        G = np.dot(xa.T, xa)
        Gi = np.linalg.inv(G)
        A = np.sum(Gi)**(-0.5)

        # equation 2.6
        w = np.sum(A * Gi, axis=1)
        u = np.dot(xa, w)

        # equation 2.11
        a = np.dot(xarr.T, u)

        # equation 2.13
        g1 = (C - c[inactive]) / (A - a[inactive])
        g2 = (C + c[inactive]) / (A + a[inactive])
        g = np.concatenate((g1, g2))
        g = g[g > 0.0]

        if g.shape[0] == 0:
            gammahat = C / A # page 9
        else:
            gammahat = np.min(g)

        beta[active] = beta[active] + gammahat * w
        est[i+1, active] = beta[active] * s
        mu = mu + (gammahat * u) # equation 2.12

    return np.asarray(active), est, maxsteps


class LARS():
    """Least Angle Regression.
    """

    def __init__(self, maxsteps=None):
        """Initialization.

        :Parameters:
          maxsteps : int (> 0) or None
            maximum number of steps.
        """

        if maxsteps is not None:
            if maxsteps <= 0:
                raise ValueError("maxsteps must be > 0")

        self._maxsteps = maxsteps

        self._beta = None
        self._beta0 = None
        self._est = None
        self._active = None
        self._steps = None

    def learn(self, x, y):
        """Compute the regression coefficients.

        :Parameters:
          x : 2d array_like object (N x P)
            matrix of regressors
          y : 1d array_like object (N)
            response
        """

        xarr = np.array(x, dtype=np.float, copy=True)
        yarr = np.array(y, dtype=np.float, copy=True)

        if xarr.ndim != 2:
            raise ValueError("x must be a 2d array_like object")

        if yarr.ndim != 1:
            raise ValueError("y must be an 1d array_like object")

        if xarr.shape[0] != yarr.shape[0]:
            raise ValueError("x, y shape mismatch")

        # center x
        xmean = np.mean(xarr, axis=0)
        xarr -= xmean

        # normalize x
        xnorm = np.sqrt(np.sum((xarr)**2, axis=0))
        xarr /= xnorm

        # center y
        ymean = np.mean(yarr)

        self._active, self._est, self._steps = \
            lars_base(xarr, yarr, self._maxsteps)
        self._est /= xnorm
        self._beta = np.copy(self._est[-1])
        self._beta0 = ymean - np.dot(xmean, self._beta)

    def pred(self, t):
        """Compute the predicted response.

        :Parameters:
           t : 1d or 2d array_like object ([M,] P)
              test data

        :Returns:
           p : float or 1d numpy array
              predicted response
        """
        if not self._beta or not self._beta0:
            raise ValueError('no mode computed; run learn() first')

        tarr = np.asarray(t, dtype=np.float)

        if tarr.ndim > 2 or tarr.ndim < 1:
            raise ValueError("t must be an 1d or a 2d array_like object")

        try:
            p = np.dot(tarr, self._beta) + self._beta0
        except ValueError:
            raise ValueError("t, beta: shape mismatch")

        return p

    def est(self):
        """Returns all LARS estimates.
        """

        return self._est

    def active(self):
        """Returns the active features.
        """

        return self._active

    def beta(self):
        """Return b_1, ..., b_p.
        """

        return self._beta

    def beta0(self):
        """Return b_0.
        """

        return self._beta0

    def steps(self):
        """Return the number of steps performed.
        """

        return self._steps



cdef extern from "clcs.h":
    
    ctypedef struct Path:
       int k
       int *px
       int *py
       
    
    int std(long *x, long *y, char **b, int n, int m)
    int real(double *x, double *y, char **b, int n, int m, double eps, int delta)
    void trace(char **b, int n, int m, Path *p)



cdef extern from "libsvm/svm.h":
    
    cdef struct svm_node:
        int index
        double value
    
    cdef struct svm_problem:
        int l
        double *y
        svm_node **x
    
    cdef struct svm_parameter:
        int svm_type
        int kernel_type
        int degree	# for poly
        double gamma	# for poly/rbf/sigmoid
        double coef0	# for poly/sigmoid

        # these are for training only
        double cache_size # in MB
        double eps	# stopping criteria
        double C	# for C_SVC, EPSILON_SVR, and NU_SVR
        int nr_weight	      # for C_SVC
        int *weight_label     # for C_SVC
        double* weight	      # for C_SVC
        double nu	# for NU_SVC, ONE_CLASS, and NU_SVR
        double p	# for EPSILON_SVR
        int shrinking	# use the shrinking heuristics
        int probability # do probability estimates

    cdef struct svm_model:
        svm_parameter param	# parameter
        int nr_class	# number of classes, = 2 in regression/one class svm
        int l	        # total SV
        svm_node **SV	# SVs (SV[l])
        # coefficients for SVs in decision functions (sv_coef[k-1][l])
        double **sv_coef 
        double *rho     # constants in decision functions (rho[k*(k-1)/2])
        # pairwise probability information
        double *probA
        double *probB
        
        # for classification only
        int *label    # label of each class (label[k])
        int *nSV      # number of SVs for each class (nSV[k])
                      # nSV[0] + nSV[1] + ... + nSV[k-1] = l
        
        # XXX
        int free_sv  # 1 if svm_model is created by svm_load_model
                     # 0 if svm_model is created by svm_train

    void svm_set_print_string_function(void (*print_func)(char *))
        
    svm_model *svm_train(svm_problem *prob, svm_parameter *param)

    double svm_predict(svm_model *model, svm_node *x)
    double svm_predict_probability(svm_model *model, 
        svm_node *x, double* prob_estimates)
    double svm_predict_values(svm_model *model, 
        svm_node *x, double* dec_values)
    
    void svm_free_and_destroy_model(svm_model **model_ptr_ptr)
    void svm_destroy_param(svm_parameter* param)
    
    int svm_save_model(char *model_file_name, svm_model *model)
    svm_model *svm_load_model(char *model_file_name)
    
    char *svm_check_parameter(svm_problem *prob, svm_parameter *param)
    int svm_check_probability_model(svm_model *model)

    
